#!/usr/bin/env node
'use strict';

var path = require('path');
var chalk = require('chalk');
var osLocale = require('os-locale');
var Linker = require('../linker');
var MessageWriter = require('../libs/message-writer.js');

/**
 * uoTES.js コンパイラモジュール
 * リンクコマンド　メインクラス
 * 
 * @since 2016/09/15
 * @author RaTTiE
 */
class Main {
	/**
	 * コンストラクタ。
	 */
	constructor() {
		this.commander = require('commander');
		this.linker = new Linker();
	}

	/**
	 * 処理を開始する。
	 */
	start() {
		if (!this.setup()) return;
		if (!this.link()) return;
	}

	/**
	 * セットアップを行う。
	 */
	setup() {
		if (!this.setupCommander()) return false;
		if (!this.setupResources()) return false;
		if (!this.setupEngine()) return false;

		return true;
	}

	/**
	 * Commanderのセットアップを行う。
	 */
	setupCommander() {
		this.commander
			.version(this.linker.version)
			.usage('[options] inFile, outFile')
			.option('-m, --modules <modules>', 'assign modules', String)
			.parse(process.argv);

		return true;
	}

	/**
	 * アプリケーションリソースのセットアップ処理を行う。
	 */
	setupResources() {
		this.linker.addResourcePath(path.resolve(__dirname, '../resources'));
		this.linker.language = osLocale.sync();

		this.linker.loadStringResource();

		return true;
	}

	/**
	 * エンジンのセットアップを行う。
	 */
	setupEngine() {
		var modules = ['mv'];

		if (this.commander.modules) {
			modules = this.commander.modules.split(',');
		}

		try {
			this.linker.loadModules(modules);
		} catch (err) {
			console.error(chalk.bold.red(err.stack));

			return false;
		}
		this.linker.messageWriter = new MessageWriter(this.linker.stringResource);

		return true;
	}

	/**
	 * TESイベントファイルのリンク処理を行う。
	 */
	link() {
		if (this.commander.args.length < 2) {
			this.commander.help();

			return false;
		}

		try {
			return this.linker.execute(
				this.commander.args[0],
				this.commander.args[1]
			);
		} catch (err) {
			console.error(chalk.bold.red(err.stack));

			return false;
		}
	}
}

var main = new Main();

main.start();
