'use strict';

var chalk = require('chalk');

/**
 * uoTES.js リンカーモジュール
 * メッセージライタークラス
 * 
 * @since 2016/09/15
 * @author RaTTiE
 */
class MessageWriter {
	/**
	 * コンストラクタ。
	 */
	constructor(stringResource) {
		this.stringResource = stringResource;
		this.fileName = null;
	}

	/**
	 * メッセージを出力する。
	 */
	writeMessage(message, code, args, address) {
		console.log(this.getLogHeader(address) + this.stringResource.get(code, args));
	}

	/**
	 * エラーメッセージを出力する。
	 */
	writeErrorMessage(message, code, args, address) {
		console.error(chalk.bold.yellow(this.getLogHeader(address) + this.stringResource.get(code, args)));
	}

	/**
	 * 警告メッセージを出力する。
	 */
	writeWarningMessage(message, code, args, address) {
		console.warn(chalk.bold.blue(this.getLogHeader(address) + this.stringResource.get(code, args)));
	}

	/**
	 * エラーを出力する。
	 */
	writeError(err, address) {
		var header = this.getLogHeader(address);

		if (err.code) {
			console.error(chalk.bold.yellow(header + this.stringResource.get(err.code, err.args)));
		} else {
			console.error(chalk.bold.yellow(header + err.message));
			console.error(chalk.bold.red(err.stack));
		}
	}

	/**
	 * ログヘッダを出力する。
	 */
	getLogHeader(address) {
		if (address != null) {
			return this.fileName + '(' + address.toString() + ') : ';
		} else {
			return this.fileName + ' : ';
		}
	}
}

module.exports = MessageWriter;
