'use strict';

(function(definition) {
if (typeof exports === 'object') {
	// CommonJS
	module.exports = definition();
} else {
	// <script>
	if (Tes.module == null) Tes.module = {};

	Tes.module.mv = definition();
}
})(function() {
// TESの注入
var Tes;
if (typeof exports === 'object') {
	// CommonJS
	Tes = require('uotes-core');
} else {
	// <script>
	Tes = window.Tes;
}

// モジュールオブジェクト作成
var module =  {
	splitFuncs: {},
	validates: {},
	validateFuncs: {},
	convertFuncs: {},
	preProcessors: {},
	processors: {},
	postProcessors: {},
	stringResourceData: {},
	deconvertFuncs: {},
	linkFuncs: {}
};

/*******************************************************************************
 * バリデーション関数
 ******************************************************************************/
// 変数値チェック
module.validateFuncs['varCheck'] = function(vargs, paramName, params) {
	if (params[paramName] == null) return;

	var $ = params[paramName].match(vargs[0]);

	// 変数指定ではない
	if ($ == null) return;

	if ($[vargs[1]] != null) {
		if (parseInt($[vargs[2]], 0) == 0) {
			throw new Tes.ValidateError(
				'[' + paramName + '] : 「0」が使われました。',
				'validator.zeroCheck.error',
				[paramName]
			);
		}
	}
};

// 変数値チェック２
module.validateFuncs['varCheck2'] = function(vargs, paramName, params) {
	if (params[paramName] == null) return;

	// 変数指定ではない
	if (params[vargs[0]] != 'var') return;

	if (parseInt(params[paramName], 10) == 0) {
		throw new Tes.ValidateError(
			'[' + paramName + '] : 「0」が使われました。',
			'validator.zeroCheck.error',
			[paramName]
		);
	}
};

/*******************************************************************************
 * 汎用処理
 ******************************************************************************/
var opList = ['=', '+', '-', '*', '/', '%'];
var flagList = ['on', 'off'];
var varRegCheck = [/^[-+]{0,1}(var\.){0,1}\d+$/];
var varVarCheck = [/^[-+]{0,1}(var\.){0,1}(\d+)$/, 1, 2];

/**
 * 変数操作形式のタイプを取得し、文字列の置換を行う
 * 
 * @param params パラメータオブジェクト
 */
function convertVarType(params) {
	var regexp = new RegExp('(var\.)', 'g');

	if (regexp.test(params.value)) {
		params.value = params.value.replace(regexp, '');
		params.type = 1;
	} else {
		params.type = 0;
	}
}

/**
 * 変数操作形式のタイプと値を逆変換する。
 */
function deconvertVarType(argName, args, type) {
	if (type == 1) {
		args[argName] = 'var.' + args[argName].toString();
	}
}

/**
 * 変数操作形式の符号を取得し、文字列の置換を行う
 *
 * @param params パラメータオブジェクト。
 */
function convertVerOperate(params) {
	var regexp = new RegExp('(^-)', 'g');

	if (regexp.test(params.value)) {
		params.value = params.value.replace(regexp, '');
		params.operate = 1;
	} else {
		params.operate = 0;
	}
}

/**
 * 変数操作形式の符号から、値を逆変換する。
 */
function deconvertVarOperate(argName, args, operate) {
	if (operate == 1) {
		args[argName] = '-' + args[argName].toString();
	}
}

/**
 * ルートオブジェクトを逆変換する。
 * 
 * @param repeatName repeatのパラメータ名。
 * @param skipName skipのパラメータ名。
 * @param waitName waitのパラメータ名。
 * @param args 引数オブジェクト。
 * @param route ルートオブジェクト。
 */
function deconvertRoute(repeatName, skipName, waitName, args, route) {
	if (route == null) {
		throw new Tes.DeconvertError(
			'ルートオブジェクトが未設定です。',
			'deconverter.route.undefined.error'
		);
	}

	if (route.repeat == null || typeof(route.repeat) != 'boolean') {
		throw new Tes.DeconvertError(
			'[' + repeatName + '] : 不正な値が指定されました。',
			'deconverter.undefined.error',
			[repeatName]
		);
	}
	args[repeatName] = route.repeat;

	if (route.skippable == null || typeof(route.skippable) != 'boolean') {
		throw new Tes.DeconvertError(
			'[' + skipName + '] : 不正な値が指定されました。',
			'deconverter.undefined.error',
			[skipName]
		);
	}
	args[skipName] = route.skippable;

	if (route.wait == null || typeof(route.wait) != 'boolean') {
		throw new Tes.DeconvertError(
			'[' + waitName + '] : 不正な値が指定されました。',
			'deconverter.undefined.error',
			[waitName]
		);
	}
	args[waitName] = route.wait;
}

function isArray(value) {
	return (
		value &&
		typeof value === 'object' &&
		typeof value.length === 'number' &&
		typeof value.splice === 'function' &&
		!(value.propertyIsEnumerable('length'))
	);
}
/*******************************************************************************
 * メッセージ
 ******************************************************************************/
// -----------------------------------------------------------------------------
// @message_h/101:文章の表示(ヘッダ)
// -----------------------------------------------------------------------------
module.validates['message_h'] = {
	index: {isNumeric: [0, 7]},
	back: {
		list: [
			'0', '1', '2',
			'normal', 'dark', 'transparent'
		]
	},
	pos: {
		list: [
			'0', '1', '2',
			'top', 'middle', 'bottom'
		]
	}
};
module.convertFuncs['message_h'] = function(args) {
	var back = this.convertList(args['back'], {
		'0': 0,
		'1': 1,
		'2': 2,
		'normal': 0,
		'dark': 1,
		'transparent': 2
	}, 0);
	var pos = this.convertList(args['pos'], {
		'0': 0,
		'1': 1,
		'2': 2,
		'top': 0,
		'middle': 1,
		'bottom': 2
	}, 2);

	return {
		code: 101,
		indent: this._indent,
		parameters: [
			args['actor'] || '',
			parseInt(args['index'] || 0, 10),
			back,
			pos
		]
	};
};
module.deconvertFuncs[101] = function(event) {
	var args = {};

	this.deconvertString('actor', args, event.parameters[0], '');
	this.deconvertNumeric('index', args, event.parameters[1]);
	this.deconvertList('back', args, event.parameters[2], [
		0, 'normal',
		1, 'dark',
		2, 'transparent'
	], 0);
	this.deconvertList('pos', args, event.parameters[3], [
		0, 'top',
		1, 'middle',
		2, 'bottom'
	], 0);

	return { name: 'message_h', args: args };
};

// -----------------------------------------------------------------------------
// @message/401:文章の表示
// -----------------------------------------------------------------------------
module.validates['message'] = {
};
module.convertFuncs['message'] = function(args) {
	return {
		code: 401,
		indent: this._indent,
		parameters: [
			args['value'] || ''
		]
	};
};
module.deconvertFuncs[401] = function(event) {
	var message = this.deconvertMessage('message', event.parameters[0]);
	if (message == '') message = '_';

	return message;
};

// -----------------------------------------------------------------------------
// @choice_h/102:選択肢の表示(ヘッダ)
// -----------------------------------------------------------------------------
module.validates['choice_h'] = {
	cancel: {
		list: [
			'-2', '-1',
			'tag', 'disable',
			'1', '2', '3', '4', '5', '6'
		]
	},
	default: {
		list: [
			'-1',
			'disable',
			'1', '2', '3', '4', '5', '6'
		]
	},
	pos: {
		list: [
			'0', '1', '2',
			'left', 'center', 'right'
		]
	},
	back: {
		list: [
			'0', '1', '2',
			'normal', 'dark', 'transparent'
		]
	}
};
module.splitFuncs['choice_h'] = function(text, tag) {
	var arrayArgs = {};
	var args = this.getCommandArgs(text, arrayArgs);
	var temp = {
		labels: arrayArgs.label
	};

	if (args.cancel !== undefined) temp.cancel = args.cancel;

	this.addData(tag, temp);
};
module.convertFuncs['choice_h'] = function(args) {
	var labels = args['labels'];
	if (labels == null || labels.length == 0) {
		throw new Tes.ConvertError(
			'ラベルが１つも指示されていません。',
			'converter.choice.label.empty.error'
		);
	}

	var cancel = this.convertList(args['cancel'], {
		'-2': -2,
		'-1': -1,
		'tag': -2,
		'disable': -1,
		'1': 0,
		'2': 1,
		'3': 2,
		'4': 3,
		'5': 4,
		'6': 5
	}, -1);
	var defaultChoice = this.convertList(args['default'], {
		'-1': -1,
		'disable': -1,
		'1': 0,
		'2': 1,
		'3': 2,
		'4': 3,
		'5': 4,
		'6': 5
	}, 0);
	var pos = this.convertList(args['pos'], {
		'0': 0,
		'1': 1,
		'2': 2,
		'left': 0,
		'center': 1,
		'right': 2
	}, 2);
	var back = this.convertList(args['back'], {
		'0': 0,
		'1': 1,
		'2': 2,
		'normal': 0,
		'dark': 1,
		'transparent': 2
	}, 0);

	return {
		code: 102,
		indent: this._indent,
		parameters: [
			labels,
			cancel,
			defaultChoice,
			pos,
			back
		]
	};
};
module.deconvertFuncs[102] = function(event) {
	var args = {};

	this.deconvertArray('label', args, event.parameters[0]);
	this.deconvertList('cancel', args, event.parameters[1], [
		-2, 'tag',
		-1, 'disable',
		0, '1',
		1, '2',
		2, '3',
		3, '4',
		4, '5',
		5, '6'
	]);
	this.deconvertList('default', args, event.parameters[2], [
		-1, 'disable',
		0, '1',
		1, '2',
		2, '3',
		3, '4',
		4, '5',
		5, '6'
	], 0);
	this.deconvertList('pos', args, event.parameters[3], [
		0, 'left',
		1, 'center',
		2, 'right'
	], 0);
	this.deconvertList('back', args, event.parameters[4], [
		0, 'normal',
		1, 'dark',
		2, 'transparent'
	], 0);

	return { name: 'choice_h', args: args };
};

// -----------------------------------------------------------------------------
// @choice_if/402:選択肢の表示(分岐)
// -----------------------------------------------------------------------------
module.validates['choice_if'] = {
	index: {
		notEmpty: [],
		list: [
			'1', '2', '3', '4', '5', '6'
		]
	}
};
module.splitFuncs['choice_if'] = function(text, tag) {
	var last = this._data[this._data.length - 1];
	if (this._data.length == 0 || !(last.tag == 'choice_h' || last.tag == 'end')) {
		throw new Tes.SyntaxError(
			'選択肢分岐前にヘッダまたは終端タグがありません。',
			'syntax.choice.tag.notFound.error'
		);
	}

	this.splitNormal.call(this, text, tag);
};
module.convertFuncs['choice_if'] = function(args) {
	this._indent++;

	var index = this.convertList(args['index'], {
		'1': 0,
		'2': 1,
		'3': 2,
		'4': 3,
		'5': 4,
		'6': 5
	});

	return {
		code: 402,
		indent: this._indent - 1,
		parameters: [
			index
		]
	};
};
module.deconvertFuncs[402] = function(event) {
	var args = {};
	this.deconvertList('index', args, event.parameters[0], [
		0, '1',
		1, '2',
		2, '3',
		3, '4',
		4, '5',
		5, '6'
	], 0);

	return { name: 'choice_if', args: args };
};

// -----------------------------------------------------------------------------
// @choice_cancel/403:選択肢の表示(キャンセル時)
// -----------------------------------------------------------------------------
module.splitFuncs['choice_cancel'] = function(text, tag) {
	var last = this._data[this._data.length - 1];
	if (this._data.length == 0 || !(last.tag == 'choice_h' || last.tag == 'end')) {
		throw new Tes.SyntaxError(
			'選択肢分岐前にヘッダまたは終端タグがありません。',
			'syntax.choice.tag.notFound.error'
		);
	}

	this.splitNormal.call(this, text, tag);
};
module.convertFuncs['choice_cancel'] = function(args) {
	this._indent++;

	return {
		code: 403,
		indent: this._indent - 1,
		parameters: []
	};
};
module.deconvertFuncs[403] = function(event) {
	return { name: 'choice_cancel' };
};

// -----------------------------------------------------------------------------
// @choice_end/404:選択肢の表示(終了)
// -----------------------------------------------------------------------------
module.convertFuncs['choice_end'] = function(args) {
	return {
		code: 404,
		indent: this._indent,
		parameters: []
	};
};
module.deconvertFuncs[404] = function(event) {
	return { name: 'choice_end' };
};

// -----------------------------------------------------------------------------
// @input_num/103:数値入力の処理
// -----------------------------------------------------------------------------
module.validates['input_num'] = {
	var: {
		notEmpty: [],
		isNumeric: [1, null]
	},
	num: {
		notEmpty: [],
		isNumeric: [1, 8]
	}
};
module.convertFuncs['input_num'] = function(args) {
	return {
		code: 103,
		indent: this._indent,
		parameters: [
			parseInt(args['var'], 10),
			parseInt(args['num'], 10)
		]
	};
};
module.deconvertFuncs[103] = function(event) {
	var args = {};

	this.deconvertNumeric('var', args, event.parameters[0]);
	this.deconvertNumeric('num', args, event.parameters[1]);

	return { name: 'input_num', args: args };
};
// -----------------------------------------------------------------------------
// @choice_item/104:アイテム選択の処理
// -----------------------------------------------------------------------------
module.validates['choice_item'] = {
	var: {
		notEmpty: [],
		isNumeric: [1, null]
	},
	type: {
		list: [
			'1', '2', '3', '4',
			'normal', 'key_item', 'secret_a', 'secret_b'
		]
	}
};
module.convertFuncs['choice_item'] = function(args) {
	var type = this.convertList(args['type'], {
		'1': 1,
		'2': 2,
		'3': 3,
		'4': 4,
		'normal': 1,
		'key_item': 2,
		'secret_a': 3,
		'secret_b': 4
	}, 2);

	return {
		code: 104,
		indent: this._indent,
		parameters: [
			parseInt(args['var'], 10),
			type
		]
	};
};
module.deconvertFuncs[104] = function(event) {
	var args = {};

	this.deconvertNumeric('var', args, event.parameters[0]);
	this.deconvertList('type', args, event.parameters[1], [
		1, 'normal',
		2, 'key_item',
		3, 'secret_a',
		4, 'secret_b'
	]);

	return { name: 'choice_item', args: args };
};

// -----------------------------------------------------------------------------
// @scroll_h/105:文章のスクロール表示(ヘッダ)
// -----------------------------------------------------------------------------
module.validates['scroll_h'] = {
	speed: {
		notEmpty: [],
		isNumeric: [1, 8]
	},
	noskip: {
		isBool: []
	}
};
module.splitFuncs['scroll_h'] = function(text, tag) {
	this.splitNormal.call(this, text, tag);

	// 文章本体の格納
	var messages = [];
	while (!this.eoc()) {
		// メッセージ行以外が来るまで格納
		if (!this.nextMessage()) break;

		this._index++;
		text = this.getLine();
		text = this.replaceMessageUnderscore(text);

		messages.push(
			this.createData.call(
				this, 'scroll', {
					'value': this.replaceMessage(text)
				}
			)
		);
	}

	if (messages.length == 0) {
		throw new Tes.SyntaxError(
			'文章スクロールの中身がありません。',
			'syntax.scroll.content.empty.error'
		);
	}

	for (var i = 0; i < messages.length; i++) {
		this._data.push(messages[i]);
	}
};
module.convertFuncs['scroll_h'] = function(args) {
	return {
		code: 105,
		indent: this._indent,
		parameters: [
			parseInt(args['speed'], 10),
			args['noskip'] == 'true' ? true : false
		]
	};
};
module.deconvertFuncs[105] = function(event) {
	var args = {};

	this.deconvertNumeric('speed', args, event.parameters[0]);
	this.deconvertBoolean('noskip', args, event.parameters[1]);

	return { name: 'scroll_h', args: args };
};

// -----------------------------------------------------------------------------
// @scroll/405:文章のスクロール表示
// -----------------------------------------------------------------------------
module.validates['scroll'] = {
};
module.convertFuncs['scroll'] = function(args) {
	return {
		code: 405,
		indent: this._indent,
		parameters: [
			args['value'] || ''
		]
	};
};
module.deconvertFuncs[405] = function(event) {
	var message = this.deconvertMessage('scroll', event.parameters[0]);
	if (message == '') message = '_';

	return message;
};

// -----------------------------------------------------------------------------
// @scroll_end/---:文章のスクロール表示(終了)
// -----------------------------------------------------------------------------
module.convertFuncs['scroll_end'] = function(env) {
	return null;
};

/*******************************************************************************
 * ゲーム進行
 ******************************************************************************/
/**
 * 変数オペレータの変換
 *
 * @param op オペレータ文字列
 * @return オペレータコード
 */
function convertOperator(op) {
	switch (op) {
		case '=':
			return 0;
		case '+':
			return 1;
		case '-':
			return 2;
		case '*':
			return 3;
		case '/':
			return 4;
		case '%':
			return 5;
		default:
			return 0;
	}
};

/** 変数オペレータの逆変換用リスト */
var deconvertOperatorList = [
	0, '=',
	1, '+',
	2, '-',
	3, '*',
	4, '/',
	5, '%'
];

// -----------------------------------------------------------------------------
// @sw/121:スイッチの操作
// -----------------------------------------------------------------------------
module.validates['sw'] = {
	id: {
		notEmpty: [],
		isNumeric: [1, null]
	},
	end: {isNumeric: [':id', null]},
	flag: {
		notEmpty: [],
		list: flagList
	}
};
module.convertFuncs['sw'] = function(args) {
	var sNum = parseInt(args['id'], 10);
	var eNum = parseInt(args['end'] || sNum, 10);

	return {
		code: 121,
		indent: this._indent,
		parameters: [
			sNum,
			eNum,
			this.convertFlag(args['flag'])
		]
	};
};
module.deconvertFuncs[121] = function(event) {
	var args = {};

	this.deconvertNumeric('id', args, event.parameters[0]);
	this.deconvertNumeric('end', args, event.parameters[1]);
	this.deconvertFlag('flag', args, event.parameters[2]);

	return { name: 'sw', args: args };
};

// -----------------------------------------------------------------------------
// @var/122:変数の操作
// -----------------------------------------------------------------------------
module.deconvertFuncs[122] = function(event) {
	switch (event.parameters[3]) {
		case 0:
		case 1:
			// var
			return deconvert_var.call(this, event);
		case 2:
			// var_random
			return deconvert_var_random.call(this, event);
		case 3:
			switch (event.parameters[4]) {
				case 0:
					// var_item
					return deconvert_var_item.call(this, event);
				case 1:
					// var_weapon
					return deconvert_var_weapon.call(this, event);
				case 2:
					// var_armor
					return deconvert_var_armor.call(this, event);
				case 3:
					// var_actor
					return deconvert_var_actor.call(this, event);
				case 4:
					// var_enemy
					return deconvert_var_enemy.call(this, event);
				case 5:
					// var_character
					return deconvert_var_character.call(this, event);
				case 6:
					// var_party
					return deconvert_var_party.call(this, event);
				case 7:
					// var_other
					return deconvert_var_other.call(this, event);
				default:
					throw new Tes.DeconvertError(
						'[type] : 不正な値が指定されました。',
						'deconverter.undefined.error',
						['type']
					);
			}
		case 4:
			// var_script
			return deconvert_var_script.call(this, event);
		default:
			throw new Tes.DeconvertError(
				'[type] : 不正な値が指定されました。',
				'deconverter.undefined.error',
				['type']
			);
	}
};

// -----------------------------------------------------------------------------
// @var/122:変数の操作(一般)
// -----------------------------------------------------------------------------
module.validates['var'] = {
	id: {
		notEmpty: [],
		isNumeric: [1, null]
	},
	end: {isNumeric: [':id', null]},
	op: {
		notEmpty: [],
		list: opList
	},
	value: {
		notEmpty: [],
		regCheck: varRegCheck
	}
};
module.convertFuncs['var'] = function(args) {
	var sNum = parseInt(args['id'], 10);
	var eNum = parseInt(args['end'] || sNum, 10);
	var op = convertOperator(args['op']);
	var value = args['value'];
	var type = /(var\.)/g.test(value) ? 1 : 0;
	value = parseInt(value.replace(/(var\.)/g, ''), 10);

	if (type == 1 && value <= 0) {
		throw new Tes.ConvertError(
			'変数指定時に「0」以下が使われました。',
			'converter.var.value.zeroCheck.error',
			value
		);
	}

	return {
		code: 122,
		indent: this._indent,
		parameters: [
			sNum,
			eNum,
			op,
			type,
			value
		]
	};
};
var deconvert_var = function(event) {
	var args = {};

	this.deconvertNumeric('id', args, event.parameters[0]);
	this.deconvertNumeric('end', args, event.parameters[1]);
	this.deconvertList('op', args, event.parameters[2], deconvertOperatorList);
	this.deconvertNumeric('value', args, event.parameters[4]);
	deconvertVarType('value', args, event.parameters[3]);

	return { name: 'var', args: args };
};

// -----------------------------------------------------------------------------
// @var_random/122:変数の操作(乱数)
// -----------------------------------------------------------------------------
module.validates['var_random'] = {
	id: {
		notEmpty: [],
		isNumeric: [1, null]
	},
	end: {isNumeric: [':id', null]},
	op: {
		notEmpty: [],
		list: opList
	},
	min: {
		notEmpty: [],
		isNumeric: [null, null]
	},
	max: {
		notEmpty: [],
		isNumeric: [':min', null]
	}
};
module.convertFuncs['var_random'] = function(args) {
	var sNum = parseInt(args['id'], 10);
	var eNum = parseInt(args['end'] || sNum, 10);

	return {
		code: 122,
		indent: this._indent,
		parameters: [
			sNum,
			eNum,
			convertOperator(args['op']),
			2,
			parseInt(args['min'], 10),
			parseInt(args['max'], 10)
		]
	};
};
var deconvert_var_random = function(event) {
	var args = {};

	this.deconvertNumeric('id', args, event.parameters[0]);
	this.deconvertNumeric('end', args, event.parameters[1]);
	this.deconvertList('op', args, event.parameters[2], deconvertOperatorList);
	this.deconvertNumeric('min', args, event.parameters[4]);
	this.deconvertNumeric('max', args, event.parameters[5]);

	return { name: 'var_random', args: args };
};

// -----------------------------------------------------------------------------
// @var_item/122:変数の操作(アイテム)
// -----------------------------------------------------------------------------
module.validates['var_item'] = {
	id: {
		notEmpty: [],
		isNumeric: [1, null]
	},
	end: {isNumeric: [':id', null]},
	op: {
		notEmpty: [],
		list: opList
	},
	value: {
		notEmpty: [],
		isNumeric: [1, null]
	}
};
module.convertFuncs['var_item'] = function(args) {
	var sNum = parseInt(args['id'], 10);
	var eNum = parseInt(args['end'] || sNum, 10);

	return {
		code: 122,
		indent: this._indent,
		parameters: [
			sNum,
			eNum,
			convertOperator(args['op']),
			3,
			0,
			parseInt(args['value'], 10),
			0
		]
	};
};
var deconvert_var_item = function(event) {
	var args = {};

	this.deconvertNumeric('id', args, event.parameters[0]);
	this.deconvertNumeric('end', args, event.parameters[1]);
	this.deconvertList('op', args, event.parameters[2], deconvertOperatorList);
	this.deconvertNumeric('value', args, event.parameters[5]);

	return { name: 'var_item', args: args };
};

// -----------------------------------------------------------------------------
// @var_weapon/122:変数の操作(武器)
// -----------------------------------------------------------------------------
module.validates['var_weapon'] = {
	id: {
		notEmpty: [],
		isNumeric: [1, null]
	},
	end: {isNumeric: [':id', null]},
	op: {
		notEmpty: [],
		list: opList
	},
	value: {
		notEmpty: [],
		isNumeric: [1, null]
	}
};
module.convertFuncs['var_weapon'] = function(args) {
	var sNum = parseInt(args['id'], 10);
	var eNum = parseInt(args['end'] || sNum, 10);

	return {
		code: 122,
		indent: this._indent,
		parameters: [
			sNum,
			eNum,
			convertOperator(args['op']),
			3,
			1,
			parseInt(args['value'], 10),
			0
		]
	};
};
var deconvert_var_weapon = function(event) {
	var args = {};

	this.deconvertNumeric('id', args, event.parameters[0]);
	this.deconvertNumeric('end', args, event.parameters[1]);
	this.deconvertList('op', args, event.parameters[2], deconvertOperatorList);
	this.deconvertNumeric('value', args, event.parameters[5]);

	return { name: 'var_weapon', args: args };
};

// -----------------------------------------------------------------------------
// @var_armor/122:変数の操作(防具)
// -----------------------------------------------------------------------------
module.validates['var_armor'] = {
	id: {
		notEmpty: [],
		isNumeric: [1, null]
	},
	end: {isNumeric: [':id', null]},
	op: {
		notEmpty: [],
		list: opList
	},
	value: {
		notEmpty: [],
		isNumeric: [1, null]
	}
};
module.convertFuncs['var_armor'] = function(args) {
	var sNum = parseInt(args['id'], 10);
	var eNum = parseInt(args['end'] || sNum, 10);

	return {
		code: 122,
		indent: this._indent,
		parameters: [
			sNum,
			eNum,
			convertOperator(args['op']),
			3,
			2,
			parseInt(args['value'], 10),
			0
		]
	};
};
var deconvert_var_armor = function(event) {
	var args = {};

	this.deconvertNumeric('id', args, event.parameters[0]);
	this.deconvertNumeric('end', args, event.parameters[1]);
	this.deconvertList('op', args, event.parameters[2], deconvertOperatorList);
	this.deconvertNumeric('value', args, event.parameters[5]);

	return { name: 'var_armor', args: args };
};

// -----------------------------------------------------------------------------
// @var_actor/122:変数の操作(アクター)
// -----------------------------------------------------------------------------
module.validates['var_actor'] = {
	id: {
		notEmpty: [],
		isNumeric: [1, null]
	},
	end: {isNumeric: [':id', null]},
	op: {
		notEmpty: [],
		list: opList
	},
	actor: {
		notEmpty: [],
		isNumeric: [1, null]
	},
	data: {
		notEmpty: [],
		list: [
			'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '10', '11',
			'level', 'exp', 'hp', 'mp', 'maxhp',
			'maxmp', 'atk', 'def', 'matk', 'mdef', 'agi',
			'luk'
		]
	}
};
module.convertFuncs['var_actor'] = function(args) {
	var sNum = parseInt(args['id'], 10);
	var eNum = parseInt(args['end'] || sNum, 10);
	var data = this.convertList(args['data'], {
		'0': 0,
		'1': 1,
		'2': 2,
		'3': 3,
		'4': 4,
		'5': 5,
		'6': 6,
		'7': 7,
		'8': 8,
		'9': 9,
		'10': 10,
		'11': 11,
		'level': 0,
		'exp': 1,
		'hp': 2,
		'mp': 3,
		'maxhp': 4,
		'maxmp': 5,
		'atk': 6,
		'def': 7,
		'matk': 8,
		'mdef': 9,
		'agi': 10,
		'luk': 11
	});

	return {
		code: 122,
		indent: this._indent,
		parameters: [
			sNum,
			eNum,
			convertOperator(args['op']),
			3,
			3,
			parseInt(args['actor'], 10),
			data
		]
	};
};
var deconvert_var_actor = function(event) {
	var args = {};

	this.deconvertNumeric('id', args, event.parameters[0]);
	this.deconvertNumeric('end', args, event.parameters[1]);
	this.deconvertList('op', args, event.parameters[2], deconvertOperatorList);
	this.deconvertNumeric('actor', args, event.parameters[5]);
	this.deconvertList('data', args, event.parameters[6], [
		0, 'level',
		1, 'exp',
		2, 'hp',
		3, 'mp',
		4, 'maxhp',
		5, 'maxmp',
		6, 'atk',
		7, 'def',
		8, 'matk',
		9, 'mdef',
		10, 'agi',
		11, 'luk'
	]);

	return { name: 'var_actor', args: args };
};

// -----------------------------------------------------------------------------
// @var_enemy/122:変数の操作(敵キャラ)
// -----------------------------------------------------------------------------
module.validates['var_enemy'] = {
	id: {
		notEmpty: [],
		isNumeric: [1, null]
	},
	end: {isNumeric: [':id', null]},
	op: {
		notEmpty: [],
		list: opList
	},
	enemy: {
		notEmpty: [],
		isNumeric: [0, null]
	},
	data: {
		notEmpty: [],
		list: [
			'0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
			'hp', 'mp', 'maxhp', 'maxmp', 'atk',
			'def', 'matk', 'mdef', 'agi', 'luk'
		]
	}
};
module.convertFuncs['var_enemy'] = function(args) {
	var sNum = parseInt(args['id'], 10);
	var eNum = parseInt(args['end'] || sNum, 10);
	var data = this.convertList(args['data'], {
		'0': 0,
		'1': 1,
		'2': 2,
		'3': 3,
		'4': 4,
		'5': 5,
		'6': 6,
		'7': 7,
		'8': 8,
		'9': 9,
		'hp': 0,
		'mp': 1,
		'maxhp': 2,
		'maxmp': 3,
		'atk': 4,
		'def': 5,
		'matk': 6,
		'mdef': 7,
		'agi': 8,
		'luk': 9,
	});

	return {
		code: 122,
		indent: this._indent,
		parameters: [
			sNum,
			eNum,
			convertOperator(args['op']),
			3,
			4,
			parseInt(args['enemy'], 10),
			data
		]
	};
};
var deconvert_var_enemy = function(event) {
	var args = {};

	this.deconvertNumeric('id', args, event.parameters[0]);
	this.deconvertNumeric('end', args, event.parameters[1]);
	this.deconvertList('op', args, event.parameters[2], deconvertOperatorList);
	this.deconvertNumeric('actor', args, event.parameters[5]);
	this.deconvertList('data', args, event.parameters[6], [
		0, 'hp',
		1, 'mp',
		2, 'maxhp',
		3, 'maxmp',
		4, 'atk',
		5, 'def',
		6, 'matk',
		7, 'mdef',
		8, 'agi',
		9, 'luk'
	]);

	return { name: 'var_enemy', args: args };
};

// -----------------------------------------------------------------------------
// @var_character/122:変数の操作(キャラクター)
// -----------------------------------------------------------------------------
module.validates['var_character'] = {
	id: {
		notEmpty: [],
		isNumeric: [1, null]
	},
	end: {isNumeric: [':id', null]},
	op: {
		notEmpty: [],
		list: opList
	},
	character: {
		notEmpty: [],
		isNumeric: [-1, null]
	},
	data: {
		notEmpty: [],
		list: [
			'0', '1', '2', '3', '4',
			'mapx', 'mapy', 'direction', 'screenx', 'screeny'
		]
	}
};
module.convertFuncs['var_character'] = function(args) {
	var sNum = parseInt(args['id'], 10);
	var eNum = parseInt(args['end'] || sNum, 10);
	var data = this.convertList(args['data'], {
		'0': 0,
		'1': 1,
		'2': 2,
		'3': 3,
		'4': 4,
		'mapx': 0,
		'mapy': 1,
		'direction': 2,
		'screenx': 3,
		'screeny': 4
	});

	return {
		code: 122,
		indent: this._indent,
		parameters: [
			sNum,
			eNum,
			convertOperator(args['op']),
			3,
			5,
			parseInt(args['character'], 10),
			data
		]
	};
};
var deconvert_var_character = function(event) {
	var args = {};

	this.deconvertNumeric('id', args, event.parameters[0]);
	this.deconvertNumeric('end', args, event.parameters[1]);
	this.deconvertList('op', args, event.parameters[2], deconvertOperatorList);
	this.deconvertNumeric('character', args, event.parameters[5]);
	this.deconvertList('data', args, event.parameters[6], [
		0, 'mapx',
		1, 'mapy',
		2, 'direction',
		3, 'screenx',
		4, 'screeny'
	]);

	return { name: 'var_character', args: args };
};

// -----------------------------------------------------------------------------
// @var_party/122:変数の操作(パーティ)
// -----------------------------------------------------------------------------
module.validates['var_party'] = {
	id: {
		notEmpty: [],
		isNumeric: [1, null]
	},
	end: {isNumeric: [':id', null]},
	op: {
		notEmpty: [],
		list: opList
	},
	member: {
		notEmpty: [],
		isNumeric: [0, null]
	}
};
module.convertFuncs['var_party'] = function(args) {
	var sNum = parseInt(args['id'], 10);
	var eNum = parseInt(args['end'] || sNum, 10);

	return {
		code: 122,
		indent: this._indent,
		parameters: [
			sNum,
			eNum,
			convertOperator(args['op']),
			3,
			6,
			parseInt(args['member'], 10),
			0
		]
	};
};
var deconvert_var_party = function(event) {
	var args = {};

	this.deconvertNumeric('id', args, event.parameters[0]);
	this.deconvertNumeric('end', args, event.parameters[1]);
	this.deconvertList('op', args, event.parameters[2], deconvertOperatorList);
	this.deconvertNumeric('member', args, event.parameters[5]);

	return { name: 'var_party', args: args };
};

// -----------------------------------------------------------------------------
// @var_other/122:変数の操作(その他)
// -----------------------------------------------------------------------------
module.validates['var_other'] = {
	id: {
		notEmpty: [],
		isNumeric: [1, null]
	},
	end: {isNumeric: [':id', null]},
	op: {
		notEmpty: [],
		list: opList
	},
	data: {
		notEmpty: [],
		list: [
			'0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
			'mapid', 'members', 'money', 'steps', 'playtime',
			'timer', 'save_num', 'battle_num', 'win_num', 'loss_num'
		]
	}
};
module.convertFuncs['var_other'] = function(args) {
	var sNum = parseInt(args['id'], 10);
	var eNum = parseInt(args['end'] || sNum, 10);
	var data = this.convertList(args['data'], {
		'0': 0,
		'1': 1,
		'2': 2,
		'3': 3,
		'4': 4,
		'5': 5,
		'6': 6,
		'7': 7,
		'8': 8,
		'9': 9,
		'mapid': 0,
		'members': 1,
		'money': 2,
		'steps': 3,
		'playtime': 4,
		'timer': 5,
		'save_num': 6,
		'battle_num': 7,
		'win_num': 8,
		'loss_num': 9
	});

	return {
		code: 122,
		indent: this._indent,
		parameters: [
			sNum,
			eNum,
			convertOperator(args['op']),
			3,
			7,
			data,
			0
		]
	};
};
var deconvert_var_other = function(event) {
	var args = {};

	this.deconvertNumeric('id', args, event.parameters[0]);
	this.deconvertNumeric('end', args, event.parameters[1]);
	this.deconvertList('op', args, event.parameters[2], deconvertOperatorList);
	this.deconvertList('data', args, event.parameters[5], [
		0, 'mapid',
		1, 'members',
		2, 'money',
		3, 'steps',
		4, 'playtime',
		5, 'timer',
		6, 'save_num',
		7, 'battle_num',
		8, 'win_num',
		9, 'loss_num'
	]);

	return { name: 'var_other', args: args };
};

// -----------------------------------------------------------------------------
// @var_script/122:変数の操作(スクリプト)
// -----------------------------------------------------------------------------
module.validates['var_script'] = {
	id: {
		notEmpty: [],
		isNumeric: [1, null]
	},
	end: {isNumeric: [':id', null]},
	op: {
		notEmpty: [],
		list: opList
	},
	script: {notEmpty: []}
};
module.convertFuncs['var_script'] = function(args) {
	var sNum = parseInt(args['id'], 10);
	var eNum = parseInt(args['end'] || sNum, 10);

	return {
		code: 122,
		indent: this._indent,
		parameters: [
			sNum,
			eNum,
			convertOperator(args['op']),
			4,
			args['sctipt']
		]
	};
};
var deconvert_var_script = function(event) {
	var args = {};

	this.deconvertNumeric('id', args, event.parameters[0]);
	this.deconvertNumeric('end', args, event.parameters[1]);
	this.deconvertList('op', args, event.parameters[2], deconvertOperatorList);
	this.deconvertString('script', args, event.parameters[3]);

	return { name: 'var_script', args: args };
};

// -----------------------------------------------------------------------------
// @self_sw/123:セルフスイッチの操作
// -----------------------------------------------------------------------------
module.validates['self_sw'] = {
	id: {
		notEmpty: [],
		list: ['A', 'B', 'C', 'D']
	},
	flag: {
		notEmpty: [],
		list: flagList
	}
};
module.convertFuncs['self_sw'] = function(args) {
	return {
		code: 123,
		indent: this._indent,
		parameters: [
			args['id'],
			this.convertFlag(args['flag'])
		]
	};
};
module.deconvertFuncs[123] = function(event) {
	var args = {};

	this.deconvertString('id', args, event.parameters[0]);
	this.deconvertFlag('flag', args, event.parameters[1]);

	return { name: 'self_sw', args: args };
};

// -----------------------------------------------------------------------------
// @timer/124:タイマーの操作
// -----------------------------------------------------------------------------
module.validates['timer'] = {
	flag: {
		notEmpty: [],
		list: flagList
	},
	time: {isNumeric: [1, 5999]}
};
module.convertFuncs['timer'] = function(args) {
	var flag = this.convertFlag(args['flag']);
	var time = null;

	if (flag == 0) {
		if (args['time'] == null) {
			throw new Tes.ConvertError(
				'[time] : 必須引数が省略されています。',
				'default.notEmpty.error',
				['time']
			);
		}

		time = parseInt(args['time'], 10);
	}

	return {
		code: 124,
		indent: this._indent,
		parameters: [flag, time]
	};
};
module.deconvertFuncs[124] = function(event) {
	var args = {};

	this.deconvertFlag('flag', args, event.parameters[0]);
	this.deconvertNumeric('time', args, event.parameters[1], args.flag == 0 ? undefined : null);

	return { name: 'timer', args: args };
};

/*******************************************************************************
 * フロー制御
 ******************************************************************************/
// -----------------------------------------------------------------------------
// @if/111:条件分岐
// -----------------------------------------------------------------------------
module.deconvertFuncs[111] = function(event) {
	switch (event.parameters[0]) {
		case 0:
			return deconvert_if_sw.call(this, event);
		case 1:
			return deconvert_if_var.call(this, event);
		case 2:
			return deconvert_if_self_sw.call(this, event);
		case 3:
			return deconvert_if_timer.call(this, event);
		case 4:
			return deconvert_if_actor.call(this, event);
		case 5:
			return deconvert_if_enemy.call(this, event);
		case 6:
			return deconvert_if_character.call(this, event);
		case 7:
			return deconvert_if_money.call(this, event);
		case 8:
			return deconvert_if_item.call(this, event);
		case 9:
			return deconvert_if_weapon.call(this, event);
		case 10:
			return deconvert_if_armor.call(this, event);
		case 11:
			return deconvert_if_button.call(this, event);
		case 12:
			return deconvert_if_script.call(this, event);
		case 13:
			return deconvert_if_vehicle.call(this, event);
		default:
			throw new Tes.DeconvertError(
				'[type] : 不正な値が指定されました。',
				'deconverter.undefined.error',
				['type']
			);
	};
};

// -----------------------------------------------------------------------------
// @if_sw/111:条件分岐(スイッチ)
// -----------------------------------------------------------------------------
module.validates['if_sw'] = {
	id: {
		notEmpty: [],
		isNumeric: [1, null]
	},
	flag: {list: flagList}
};
module.convertFuncs['if_sw'] = function(args) {
	this._indent++;

	return {
		code: 111,
		indent: this._indent - 1,
		parameters: [
			0,
			parseInt(args['id'], 10),
			this.convertFlag(args['flag'])
		]
	};
};
var deconvert_if_sw = function(event) {
	var args = {};
	
	this.deconvertNumeric('id', args, event.parameters[1]);
	this.deconvertFlag('flag', args, event.parameters[2]);

	return { name: 'if_sw', args: args };
};

// -----------------------------------------------------------------------------
// @if_var/111:条件分岐(変数)
// -----------------------------------------------------------------------------
module.validates['if_var'] = {
	id: {
		notEmpty: [],
		isNumeric: [1, null]
	},
	value: {
		notEmpty: [],
		regCheck: varRegCheck,
		varCheck: varVarCheck
	},
	op: {list: ['=', '!=', '<>', '><', '>=', '=>', '>', '<=', '=<', '<']}
};
module.convertFuncs['if_var'] = function(args) {
	this._indent++;

	var params = {value: args['value']};

	convertVarType(params);

	if (params.type == 1 && params.value <= 0) {
		throw new Tes.ConvertError(
			'変数指定時に「0」以下が使われました。',
			'converter.var.value.zeroCheck.error',
			['value']
		);
	}

	var op = this.convertList(args['op'], {
		'=': 0,
		'>=': 1,
		'=>': 1,
		'<=': 2,
		'=<': 2,
		'>': 3,
		'<': 4,
		'!=': 5,
		'<>': 5,
		'><': 5
	}, 0);

	return {
		code: 111,
		indent: this._indent - 1,
		parameters: [
			1,
			parseInt(args['id'], 10),
			params.type,
			parseInt(params.value, 10),
			op
		]
	};
};
var deconvert_if_var = function(event) {
	var args = {};
	
	this.deconvertNumeric('id', args, event.parameters[1]);
	this.deconvertNumeric('value', args, event.parameters[3]);
	deconvertVarType('value', args, event.parameters[2]);
	this.deconvertList('op', args, event.parameters[4], [
		0, '=',
		1, '>=',
		2, '<=',
		3, '>',
		4, '<',
		5, '<>'
	]);

	return { name: 'if_var', args: args };
};

// -----------------------------------------------------------------------------
// @if_self_sw/111:条件分岐(セルフスイッチ)
// -----------------------------------------------------------------------------
module.validates['if_self_sw'] = {
	id: {
		notEmpty: [],
		list: ['A', 'B', 'C', 'D']
	},
	flag: {list: flagList}
};
module.convertFuncs['if_self_sw'] = function(args) {
	this._indent++;

	return {
		code: 111,
		indent: this._indent - 1,
		parameters: [
			2,
			args['id'],
			this.convertFlag(args['flag'])
		]
	};
};
var deconvert_if_self_sw = function(event) {
	var args = {};

	this.deconvertString('id', args, event.parameters[1]);
	this.deconvertFlag('flag', args, event.parameters[2]);

	return { name: 'if_self_sw', args: args };
};

// -----------------------------------------------------------------------------
// @if_timer/111:条件分岐(タイマー)
// -----------------------------------------------------------------------------
module.validates['if_timer'] = {
	time: {
		notEmpty: [],
		isNumeric: [0, 5999]
	},
	op: {
		list: ['>=', '=>', '<=', '=<']
	}
};
module.convertFuncs['if_timer'] = function(args) {
	this._indent++;

	var op = this.convertList(args['op'], {
		'>=': 0,
		'=>': 0,
		'<=': 1,
		'=<': 1
	}, 0);

	return {
		code: 111,
		indent: this._indent - 1,
		parameters: [
			3,
			parseInt(args['time'], 10),
			op
		]
	};
};
var deconvert_if_timer = function(event) {
	var args = {};

	this.deconvertNumeric('time', args, event.parameters[1]);
	this.deconvertList('op', args, event.parameters[2], [
		0, '>=',
		1, '<='
	]);

	return { name: 'if_timer', args: args };
};

// -----------------------------------------------------------------------------
// @if_actor/111:条件分岐(アクター)
// -----------------------------------------------------------------------------
module.validates['if_actor'] = {
	id: {
		notEmpty: [],
		isNumeric: [1, null]
	},
	type: {
		list: [
			'0', '1', '2', '3', '4', '5', '6',
			'party', 'name', 'class', 'skill', 'weapon',
			'armor', 'state'
		]
	}
};
module.convertFuncs['if_actor'] = function(args) {
	this._indent++;

	var type = this.convertList(args['type'], {
		'0': 0,
		'1': 1,
		'2': 2,
		'3': 3,
		'4': 4,
		'5': 5,
		'6': 6,
		'party': 0,
		'name': 1,
		'class': 2,
		'skill': 3,
		'weapon': 4,
		'armor': 5,
		'state': 6
	}, 0);
	var value;

	switch (type) {
		case 0:
			value = 0;

			break;
		case 1:
			value = args['value'];

			break;
		case 2:
		case 3:
		case 4:
		case 5:
		case 6:
			Tes.CommonFunctions.checkNotEmpty.call(this, 'value', args, Tes.ConvertError);
			value = Tes.CommonFunctions.checkNumeric.call(this, 'value', args, 1, null, Tes.ConvertError);

			break;
	}

	return {
		code: 111,
		indent: this._indent - 1,
		parameters: [
			4,
			parseInt(args['id'], 10),
			type,
			value
		]
	};
};
var deconvert_if_actor = function(event) {
	var args = {};

	this.deconvertNumeric('id', args, event.parameters[1]);
	this.deconvertList('type', args, event.parameters[2], [
		0, 'party',
		1, 'name',
		2, 'class',
		3, 'skill',
		4, 'weapon',
		5, 'armor',
		6, 'state'
	]);

	switch (args.type) {
		case 'party':
			break;
		case 'name':
			this.deconvertString('value', args, event.parameters[3]);

			break;
		case 'class':
		case 'skill':
		case 'weapon':
		case 'armor':
		case 'state':
			this.deconvertNumeric('value', args, event.parameters[3]);

			break;
		default:
			throw new Tes.DeconvertError(
				'[type] : 不正な値が指定されました。',
				'deconverter.undefined.error',
				['type']
			);
	}

	return { name: 'if_actor', args: args };
};

// -----------------------------------------------------------------------------
// @if_enemy/111:条件分岐(敵キャラ)
// -----------------------------------------------------------------------------
module.validates['if_enemy'] = {
	enemy: {
		notEmpty: [],
		isNumeric: [0, null]
	},
	type: {
		list: [
			'0', '1',
			'visible', 'state'
		]
	},
	value: {isNumeric: [1, null]}
};
module.convertFuncs['if_enemy'] = function(args) {
	this._indent++;

	var type = this.convertList(args['type'], {
		'0': 0,
		'1': 1,
		'visible': 0,
		'state': 1
	}, 0);
	var value;

	switch (type) {
		case 0:
			value = 0;

			break;
		case 1:
			Tes.CommonFunctions.checkNotEmpty.call(this, 'value', args, Tes.ConvertError);
			value = Tes.CommonFunctions.checkNumeric.call(this, 'value', args, 1, null, Tes.ConvertError);

			break;
	}

	return {
		code: 111,
		indent: this._indent - 1,
		parameters: [
			5,
			parseInt(args['enemy'], 10),
			type,
			value
		]
	};
};
var deconvert_if_enemy = function(event) {
	var args = {};

	this.deconvertNumeric('id', args, event.parameters[1]);
	this.deconvertList('type', args, event.parameters[2], [
		0, 'visible',
		1, 'state'
	]);
	this.deconvertNumeric('value', args, event.parameters[3]);

	return { name: 'if_enemy', args: args };
};

// -----------------------------------------------------------------------------
// @if_character/111:条件分岐(キャラクター)
// -----------------------------------------------------------------------------
module.validates['if_character'] = {
	id: {
		notEmpty: [],
		isNumeric: [-1, null]
	},
	direction: {
		notEmpty: [],
		list: [
			'2', '4', '6', '8',
			'left', 'right', 'up', 'down'
		]
	}
};
module.convertFuncs['if_character'] = function(args) {
	this._indent++;

	var direction = this.convertList(args['direction'], {
		'2': 2,
		'4': 4,
		'6': 6,
		'8': 8,
		'down': 2,
		'left': 4,
		'right': 6,
		'up': 8
	});

	return {
		code: 111,
		indent: this._indent - 1,
		parameters: [
			6,
			parseInt(args['id'], 10),
			direction
		]
	};
};
var deconvert_if_character = function(event) {
	var args = {};

	this.deconvertNumeric('id', args, event.parameters[1]);
	this.deconvertList('direction', args, event.parameters[2], [
		2, 'down',
		4, 'left',
		6, 'right',
		8, 'up'
	]);

	return { code: 'if_character', args: args };
};

// -----------------------------------------------------------------------------
// @if_vehicle/111:条件分岐(乗り物)
// -----------------------------------------------------------------------------
module.validates['if_vehicle'] = {
	vehicle: {
		notEmpty: [],
		list: [
			'0', '1', '2',
			'boat', 'ship', 'airship'
		]
	}
};
module.convertFuncs['if_vehicle'] = function(args) {
	this._indent++;

	var vehicle = this.convertList(args['vehicle'], {
		'0': 0,
		'1': 1,
		'2': 2,
		'boat': 0,
		'ship': 1,
		'airship': 2
	});

	return {
		code: 111,
		indent: this._indent - 1,
		parameters: [
			13,
			vehicle
		]
	};
};
var deconvert_if_vehicle = function(event) {
	var args = {};

	this.deconvertList('vehicle', args, event.parameters[1], [
		0, 'boat',
		1, 'ship',
		2, 'airship'
	]);

	return { name: 'if_vehicle', args: args };
};

// -----------------------------------------------------------------------------
// @if_money/111:条件分岐(お金)
// -----------------------------------------------------------------------------
module.validates['if_money'] = {
	money: {
		notEmpty: [],
		isNumeric: [0, null]
	},
	op: {
		list: ['>=', '=>', '<=', '=<', '<']
	}
};
module.convertFuncs['if_money'] = function(args) {
	this._indent++;

	var op = this.convertList(args['op'], {
		'>=': 0,
		'=>': 0,
		'<=': 1,
		'=<': 1,
		'<': 2
	}, 0)

	return {
		code: 111,
		indent: this._indent - 1,
		parameters: [
			7,
			parseInt(args['money'], 10),
			op
		]
	};
};
var deconvert_if_money = function(event) {
	var args = {};

	this.deconvertNumeric('money', args, event.parameters[1]);
	this.deconvertList('op', args, event.parameters[2], [
		0, '>=',
		1, '<=',
		2, '<'
	]);

	return { name: 'if_money', args: args };
};

// -----------------------------------------------------------------------------
// @if_item/111:条件分岐(アイテム)
// -----------------------------------------------------------------------------
module.validates['if_item'] = {
	id: {
		notEmpty: [],
		isNumeric: [1, null]
	}
};
module.convertFuncs['if_item'] = function(args) {
	this._indent++;

	return {
		code: 111,
		indent: this._indent - 1,
		parameters: [
			8,
			parseInt(args['id'], 10)
		]
	};
};
var deconvert_if_item = function(event) {
	var args = {};

	this.deconvertNumeric('id', args, event.parameters[1]);

	return { name: 'if_item', args: args };
};

// -----------------------------------------------------------------------------
// @if_weapon/111:条件分岐(武器)
// -----------------------------------------------------------------------------
module.validates['if_weapon'] = {
	id: {
		notEmpty: [],
		isNumeric: [1, null]
	},
	equip: {isBool: []}
};
module.convertFuncs['if_weapon'] = function(args) {
	this._indent++;

	return {
		code: 111,
		indent: this._indent - 1,
		parameters: [
			9,
			parseInt(args['id'], 10),
			(args['equip'] || 'false') == 'true' ? true : false
		]
	};
};
var deconvert_if_weapon = function(event) {
	var args = {};

	this.deconvertNumeric('id', args, event.parameters[1]);
	this.deconvertBoolean('equip', args, event.parameters[2]);

	return { name: 'if_weapon', args: args };
};

// -----------------------------------------------------------------------------
// @if_armor/111:条件分岐(防具)
// -----------------------------------------------------------------------------
module.validates['if_armor'] = {
	id: {
		notEmpty: [],
		isNumeric: [1, null]
	},
	equip: {isBool: []}
};
module.convertFuncs['if_armor'] = function(args) {
	this._indent++;

	return {
		code: 111,
		indent: this._indent - 1,
		parameters: [
			10,
			parseInt(args['id'], 10),
			(args['equip'] || 'false') == 'true' ? true : false
		]
	};
};
var deconvert_if_armor = function(event) {
	var args = {};

	this.deconvertNumeric('id', args, event.parameters[1]);
	this.deconvertBoolean('equip', args, event.parameters[2]);

	return { name: 'if_armor', args: args };
};

// -----------------------------------------------------------------------------
// @if_button/111:条件分岐(ボタン)
// -----------------------------------------------------------------------------
module.validates['if_button'] = {
	button: {
		notEmpty: [],
		list: [
			'A', 'B', 'X', 'L', 'R',
			'down', 'left', 'right', 'up',
			'ok', 'cancel', 'shift',
			'pageup', 'pagedown'
		]
	}
};
module.convertFuncs['if_button'] = function(args) {
	this._indent++;

	var button = this.convertList(args['button'], {
		'A': 'ok',
		'B': 'cancel',
		'X': 'shift',
		'L': 'pageup',
		'R': 'pagedown',
		'down': 'down',
		'left': 'left',
		'right': 'right',
		'up': 'up',
		'ok': 'ok',
		'cancel': 'cancel',
		'shift': 'shift',
		'pageup': 'pageup',
		'pagedown': 'pagedown'
	});

	return {
		code: 111,
		indent: this._indent - 1,
		parameters: [
			11,
			button
		]
	};
};
var deconvert_if_button = function(event) {
	var args = {};

	this.deconvertString('button', args, event.parameters[1]);

	return { name: 'if_button', args: args };
};

// -----------------------------------------------------------------------------
// @if_script/111:条件分岐(スクリプト)
// -----------------------------------------------------------------------------
module.validates['if_script'] = {
	script: {notEmpty: []}
};
module.convertFuncs['if_script'] = function(args) {
	this._indent++;

	return {
		code: 111,
		indent: this._indent - 1,
		parameters: [
			12,
			args['script']
		]
	};
};
var deconvert_if_script = function(event) {
	var args = {};

	this.deconvertString('script', args, event.parameters[1]);

	return { name: 'if_script', args: args };
};

// -----------------------------------------------------------------------------
// @else/411:条件分岐(それ以外)
// -----------------------------------------------------------------------------
module.convertFuncs['else'] = function(args) {
	this._indent++;

	return {
		code: 411,
		indent: this._indent - 1,
		parameters: []
	};
};
module.deconvertFuncs[411] = function(event) {
	return { name: 'else' };
};

// -----------------------------------------------------------------------------
// @loop/112:ループ
// -----------------------------------------------------------------------------
module.convertFuncs['loop'] = function(args) {
	this._indent++;

	return {
		code: 112,
		indent: this._indent - 1,
		parameters: []
	};
};
module.deconvertFuncs[112] = function(event) {
	return { name: 'loop' };
};

// -----------------------------------------------------------------------------
// @if_end/412:条件分岐(分岐終了)
// -----------------------------------------------------------------------------
module.convertFuncs['if_end'] = function(args) {
	return {
		code: 412,
		indent: this._indent,
		parameters: []
	}
};
module.deconvertFuncs[412] = function(event) {
	return { name: 'if_end' };
};

// -----------------------------------------------------------------------------
// @loop_end/413:ループの終わり
// -----------------------------------------------------------------------------
module.convertFuncs['loop_end'] = function(args) {
	this._indent--;

	return {
		code: 413,
		indent: this._indent,
		parameters: []
	};
};
module.deconvertFuncs[413] = function(event) {
	return { name: 'loop_end', indent: event.indent - 1 };
};

// -----------------------------------------------------------------------------
// @loop_break/113:ループの中断
// -----------------------------------------------------------------------------
module.convertFuncs['loop_break'] = function(args) {
	return {
		code: 113,
		indent: this._indent,
		parameters: []
	};
};
module.deconvertFuncs[113] = function(event) {
	return { name: 'loop_break' };
};

// -----------------------------------------------------------------------------
// @event_break/115:イベントの中断
// -----------------------------------------------------------------------------
module.convertFuncs['event_break'] = function(args) {
	return {
		code: 115,
		indent: this._indent,
		parameters: []
	};
};

// -----------------------------------------------------------------------------
// @return/115:イベントの中断
// -----------------------------------------------------------------------------
module.convertFuncs['return'] = function(args) {
	return module.convertFuncs['event_break'].call(this, args);
};
module.deconvertFuncs[115] = function(event) {
	return { name: 'return' };
};

// -----------------------------------------------------------------------------
// @common/117:コモンイベント
// -----------------------------------------------------------------------------
module.validates['common'] = {
	id: {
		notEmpty: [],
		isNumeric: [1, null]
	}
};
module.convertFuncs['common'] = function(args) {
	return {
		code: 117,
		indent: this._indent,
		parameters: [parseInt(args['id'], 10)]
	};
};
module.deconvertFuncs[117] = function(event) {
	var args = {};
	
	this.deconvertNumeric('id', args, event.parameters[0]);

	return { name: 'common', args: args };
};

// -----------------------------------------------------------------------------
// @label/118:ラベル
// -----------------------------------------------------------------------------
module.validates['label'] = {
	value: {notEmpty: []}
};
module.convertFuncs['label'] = function(args) {
	return {
		code: 118,
		indent: this._indent,
		parameters: [args['value']]
	};
};
module.deconvertFuncs[118] = function(event) {
	var args = {};

	this.deconvertString('value', args, event.parameters[0]);

	return { name: 'label', args: args };
};

// -----------------------------------------------------------------------------
// @label_jump/119:ラベルジャンプ
// -----------------------------------------------------------------------------
module.validates['label_jump'] = {
	value: {notEmpty: []}
};
module.convertFuncs['label_jump'] = function(args) {
	return {
		code: 119,
		indent: this._indent,
		parameters: [args['value']]
	};
};
module.deconvertFuncs[119] = function(event) {
	var args = {};

	this.deconvertString('value', args, event.parameters[0]);

	return { name: 'label_jump', args: args };
};

// -----------------------------------------------------------------------------
// @comment/108:注釈(ヘッダ)＆408:注釈
// -----------------------------------------------------------------------------
module.validates['comment'] = {
	value: {notEmpty: []}
};
module.convertFuncs['comment'] = function(args) {
	return {
		code: 108,
		indent: this._indent,
		parameters: [args['value']]
	}
};
module.splitFuncs['comment'] = function(text, tag) {
	var comments = [];
	this._index++;

	while(!this.eoc()) {
		text = this.getLine();
		if (this.getCommandName(text) == 'comment_end') break;

		comments.push(
			this.createData.call(
				this,
				'comment2', {
					value: text
				}
			)
		);

		this._index++;
	}

	if (this.eoc()) {
		throw new Tes.SyntaxError(
			'コメントタグが閉じられていません。',
			'syntax.comment.footer.notFound.error'
		);
	}

	if (comments.length > 0) {
		// 先頭のタグ名修正
		comments[0].line = 'comment';

		for (var i = 0; i < comments.length; i++) {
			this._data.push(comments[i]);
		}
	}
};
module.deconvertFuncs[108] = function(event) {
	var args = {};

	this.deconvertString('value', args, event.parameters[0]);

	return { name: 'comment', args: args };
};
module.validates['comment2'] = {
};
module.convertFuncs['comment2'] = function(args) {
	return {
		code: 408,
		indent: this._indent,
		parameters: [args['value']]
	};
};
module.deconvertFuncs[408] = function(event) {
	var args = {};

	this.deconvertString('value', args, event.parameters[0]);

	return { name: 'comment', args: args };
};

// -----------------------------------------------------------------------------
// @comment_end/---:注釈の終了
// -----------------------------------------------------------------------------
module.splitFuncs['comment_end'] = function(text, tag) {
	throw new Tes.SyntaxError(
		'コメントタグが開始されていません。',
		'syntax.comment.header.notFound.error'
	);
};

/*******************************************************************************
 * パーティ
 ******************************************************************************/
// -----------------------------------------------------------------------------
// @money/125:所持金の増減
// -----------------------------------------------------------------------------
module.validates['money'] = {
	value: {
		notEmpty: [],
		regCheck: varRegCheck,
		varCheck: varVarCheck
	}
};
module.convertFuncs['money'] = function(args) {
	var params = {value: args['value']};

	convertVarType(params);
	convertVerOperate(params);

	return {
		code: 125,
		indent: this._indent,
		parameters: [
			params.operate,
			params.type,
			parseInt(params.value, 10)
		]
	};
};
module.deconvertFuncs[125] = function(event) {
	var args = {};

	this.deconvertNumeric('value', args, event.parameters[2]);
	deconvertVarType('value', args, event.parameters[1]);
	deconvertVarOperate('value', args, event.parameters[0]);

	return { name: 'money', args: args };
};

// -----------------------------------------------------------------------------
// @item/126:アイテムの増減
// -----------------------------------------------------------------------------
module.validates['item'] = {
	id: {
		notEmpty: [],
		isNumeric: [1, null]
	},
	value: {
		notEmpty: [],
		regCheck: varRegCheck,
		varCheck: varVarCheck
	}
};
module.convertFuncs['item'] = function(args) {
	var params = {value: args['value']};

	convertVarType(params);
	convertVerOperate(params);

	return {
		code: 126,
		indent: this._indent,
		parameters: [
			parseInt(args['id'], 10),
			params.operate,
			params.type,
			parseInt(params.value, 10)
		]
	};
};
module.deconvertFuncs[126] = function(event) {
	var args = {};

	this.deconvertNumeric('id', args, event.parameters[0]);
	this.deconvertNumeric('value', args, event.parameters[3]);
	deconvertVarOperate('value', args, event.parameters[1]);
	deconvertVarType('value', args, event.parameters[2]);

	return { name: 'item', args: args };
};

// -----------------------------------------------------------------------------
// @weapon/127:武器の増減
// -----------------------------------------------------------------------------
module.validates['weapon'] = {
	id: {
		notEmpty: [],
		isNumeric: [1, null]
	},
	value: {
		notEmpty: [],
		regCheck: varRegCheck,
		varCheck: varVarCheck
	},
	equip: {isBool: []}
};
module.convertFuncs['weapon'] = function(args) {
	var params = {value: args['value']};

	convertVarType(params);
	convertVerOperate(params);

	return {
		code: 127,
		indent: this._indent,
		parameters: [
			parseInt(args['id'], 10),
			params.operate,
			params.type,
			parseInt(params.value, 10),
			(args['equip'] || 'false') == 'true' ? true : false
		]
	};
};
module.deconvertFuncs[127] = function(event) {
	var args = {};

	this.deconvertNumeric('id', args, event.parameters[0]);
	this.deconvertNumeric('value', args, event.parameters[3]);
	deconvertVarOperate('value', args, event.parameters[1]);
	deconvertVarType('value', args, event.parameters[2]);
	this.deconvertBoolean('equip', args, event.parameters[4]);

	return { name: 'weapon', args: args };
};

// -----------------------------------------------------------------------------
// @armor/128:防具の増減
// -----------------------------------------------------------------------------
module.validates['armor'] = {
	id: {
		notEmpty: [],
		isNumeric: [1, null]
	},
	value: {
		notEmpty: [],
		regCheck: varRegCheck,
		varCheck: varVarCheck
	},
	equip: {isBool: []}
};
module.convertFuncs['armor'] = function(args) {
	var params = {value: args['value']};

	convertVarType(params);
	convertVerOperate(params);

	return {
		code: 128,
		indent: this._indent,
		parameters: [
			parseInt(args['id'], 10),
			params.operate,
			params.type,
			parseInt(params.value, 10),
			(args['equip'] || 'false') == 'true' ? true : false
		]
	};
};
module.deconvertFuncs[128] = function(event) {
	var args = {};

	this.deconvertNumeric('id', args, event.parameters[0]);
	this.deconvertNumeric('value', args, event.parameters[3]);
	deconvertVarOperate('value', args, event.parameters[1]);
	deconvertVarType('value', args, event.parameters[2]);
	this.deconvertBoolean('equip', args, event.parameters[4]);

	return { name: 'armor', args: args };
};

// -----------------------------------------------------------------------------
// @member/129:メンバーの入れ替え
// -----------------------------------------------------------------------------
module.validates['member'] = {
	id: {
		notEmpty: [],
		isNumeric: [null, null],
		varCheck: [/^[-+]{0,1}(\d+)$/, 0, 1]
	},
	init: {
		isBool: []
	}
};
module.convertFuncs['member'] = function(args) {
	var params = {value: args['id']};

	convertVerOperate(params);

	return {
		code: 129,
		indent: this._indent,
		parameters: [
			parseInt(params.value, 10),
			params.operate,
			(args['init'] || 'false') == 'true' ? 1 : 0
		]
	};
};
module.deconvertFuncs[129] = function(event) {
	var args = {};

	this.deconvertNumeric('value', args, event.parameters[0]);
	deconvertVarOperate('value', args, event.parameters[1]);
	this.deconvertBoolean('init', args, event.parameters[2]);

	return { name: 'member', args: args };
};

/*******************************************************************************
 * アクター
 ******************************************************************************/
// -----------------------------------------------------------------------------
// @hp/311:HPの増減
// -----------------------------------------------------------------------------
module.validates['hp'] = {
	actor: {
		notEmpty: [],
		regCheck: varRegCheck,
		varCheck: varVarCheck
	},
	value: {
		notEmpty: [],
		regCheck: varRegCheck,
		varCheck: varVarCheck
	},
	death: {isBool: []}
};
module.convertFuncs['hp'] = function(args) {
	var actor = {value: args['actor']};
	var params = {value: args['value']};

	convertVarType(actor);
	convertVarType(params);
	convertVerOperate(params);

	return {
		code: 311,
		indent: this._indent,
		parameters: [
			actor.type,
			parseInt(actor.value, 10),
			params.operate,
			params.type,
			parseInt(params.value, 10),
			(args['death'] || 'false') == 'true' ? true : false
		]
	};
};
module.deconvertFuncs[311] = function(event) {
	var args = {};

	this.deconvertNumeric('actor', args, event.parameters[1]);
	deconvertVarType('actor', args, event.parameters[0]);
	this.deconvertNumeric('value', args, event.parameters[4]);
	deconvertVarOperate('value', args, event.parameters[2]);
	deconvertVarType('value', args, event.parameters[3]);
	this.deconvertBoolean('death', args, event.parameters[5]);

	return { name: 'hp', args: args };
};

// -----------------------------------------------------------------------------
// @mp/312:MPの増減
// -----------------------------------------------------------------------------
module.validates['mp'] = {
	actor: {
		notEmpty: [],
		regCheck: varRegCheck,
		varCheck: varVarCheck
	},
	value: {
		notEmpty: [],
		regCheck: varRegCheck,
		varCheck: varVarCheck
	}
};
module.convertFuncs['mp'] = function(args) {
	var actor = {value: args['actor']};
	var params = {value: args['value']};

	convertVarType(actor);
	convertVarType(params);
	convertVerOperate(params);

	return {
		code: 312,
		indent: this._indent,
		parameters: [
			actor.type,
			parseInt(actor.value, 10),
			params.operate,
			params.type,
			parseInt(params.value, 10)
		]
	};
};
module.deconvertFuncs[312] = function(event) {
	var args = {};

	this.deconvertNumeric('actor', args, event.parameters[1]);
	deconvertVarType('actor', args, event.parameters[0]);
	this.deconvertNumeric('value', args, event.parameters[4]);
	deconvertVarOperate('value', args, event.parameters[2]);
	deconvertVarType('value', args, event.parameters[3]);

	return { name: 'mp', args: args };
};

// -----------------------------------------------------------------------------
// @tp/326:TPの増減
// -----------------------------------------------------------------------------
module.validates['tp'] = {
	actor: {
		notEmpty: [],
		regCheck: varRegCheck,
		varCheck: varVarCheck
	},
	value: {
		notEmpty: [],
		regCheck: varRegCheck,
		varCheck: varVarCheck
	}
};
module.convertFuncs['tp'] = function(args) {
	var actor = {value: args['actor']};
	var params = {value: args['value']};

	convertVarType(actor);
	convertVarType(params);
	convertVerOperate(params);

	return {
		code: 326,
		indent: this._indent,
		parameters: [
			actor.type,
			parseInt(actor.value, 10),
			params.operate,
			params.type,
			parseInt(params.value, 10)
		]
	};
};
module.deconvertFuncs[326] = function(event) {
	var args = {};

	this.deconvertNumeric('actor', args, event.parameters[1]);
	deconvertVarType('actor', args, event.parameters[0]);
	this.deconvertNumeric('value', args, event.parameters[4]);
	deconvertVarOperate('value', args, event.parameters[2]);
	deconvertVarType('value', args, event.parameters[3]);

	return { name: 'tp', args: args };
};

// -----------------------------------------------------------------------------
// @state/313:ステートの変更
// -----------------------------------------------------------------------------
module.validates['state'] = {
	actor: {
		notEmpty: [],
		regCheck: varRegCheck,
		varCheck: varVarCheck
	},
	value: {
		notEmpty: [],
		isNumeric: [null, null],
		zeroCheck: []
	}
};
module.convertFuncs['state'] = function(args) {
	var actor = {value: args['actor']};
	var params = {value: args['value']};

	convertVarType(actor);
	convertVerOperate(params);

	return {
		code: 313,
		indent: this._indent,
		parameters: [
			actor.type,
			parseInt(actor.value, 10),
			params.operate,
			parseInt(params.value, 10)
		]
	};
};
module.deconvertFuncs[313] = function(event) {
	var args = {};

	this.deconvertNumeric('actor', args, event.parameters[1]);
	deconvertVarType('actor', args, event.parameters[0]);
	this.deconvertNumeric('value', args, event.parameters[3]);
	deconvertVarOperate('value', args, event.parameters[2]);

	return { name: 'state', args: args };
};

// -----------------------------------------------------------------------------
// @all_recovery/314:全回復
// -----------------------------------------------------------------------------
module.validates['all_recovery'] = {
	actor: {
		notEmpty: [],
		regCheck: varRegCheck,
		varCheck: varVarCheck
	}
};
module.convertFuncs['all_recovery'] = function(args) {
	var actor = {value: args['actor']};

	convertVarType(actor);

	return {
		code: 314,
		indent: this._indent,
		parameters: [
			actor.type,
			parseInt(actor.value, 10)
		]
	};
};
module.deconvertFuncs[314] = function(event) {
	var args = {};

	this.deconvertNumeric('actor', args, event.parameters[1]);
	deconvertVarType('actor', args, event.parameters[0]);

	return { name: 'all_recovery', args: args };
};

// -----------------------------------------------------------------------------
// @exp/315:経験値の増減
// -----------------------------------------------------------------------------
module.validates['exp'] = {
	actor: {
		notEmpty: [],
		regCheck: varRegCheck,
		varCheck: varVarCheck
	},
	value: {
		notEmpty: [],
		regCheck: varRegCheck,
		varCheck: varVarCheck
	},
	message: {isBool: []}
};
module.convertFuncs['exp'] = function(args) {
	var actor = {value: args['actor']};
	var params = {value: args['value']};

	convertVarType(actor);
	convertVarType(params);
	convertVerOperate(params);

	return {
		code: 315,
		indent: this._indent,
		parameters: [
			actor.type,
			parseInt(actor.value, 10),
			params.operate,
			params.type,
			parseInt(params.value, 10),
			(args['message'] || 'false') == 'true' ? true : false
		]
	};
};
module.deconvertFuncs[315] = function(event) {
	var args = {};

	this.deconvertNumeric('actor', args, event.parameters[1]);
	deconvertVarType('actor', args, event.parameters[0]);
	this.deconvertNumeric('value', args, event.parameters[4]);
	deconvertVarOperate('value', args, event.parameters[2]);
	deconvertVarType('value', args, event.parameters[3]);
	this.deconvertBoolean('message', args, event.parameters[5]);

	return { name: 'exp', args: args };
};

// -----------------------------------------------------------------------------
// @level/316:レベルの増減
// -----------------------------------------------------------------------------
module.validates['level'] = {
	actor: {
		notEmpty: [],
		regCheck: varRegCheck,
		varCheck: varVarCheck
	},
	value: {
		notEmpty: [],
		regCheck: varRegCheck,
		varCheck: varVarCheck
	},
	message: {isBool: []}
};
module.convertFuncs['level'] = function(args) {
	var actor = {value: args['actor']};
	var params = {value: args['value']};

	convertVarType(actor);
	convertVarType(params);
	convertVerOperate(params);

	return {
		code: 316,
		indent: this._indent,
		parameters: [
			actor.type,
			parseInt(actor.value, 10),
			params.operate,
			params.type,
			parseInt(params.value, 10),
			(args['message'] || 'false') == 'true' ? true : false
		]
	};
};
module.deconvertFuncs[316] = function(event) {
	var args = {};

	this.deconvertNumeric('actor', args, event.parameters[1]);
	deconvertVarType('actor', args, event.parameters[0]);
	this.deconvertNumeric('value', args, event.parameters[4]);
	deconvertVarOperate('value', args, event.parameters[2]);
	deconvertVarType('value', args, event.parameters[3]);
	this.deconvertBoolean('message', args, event.parameters[5]);

	return { name: 'level', args: args };
};

// -----------------------------------------------------------------------------
// @capability/317:能力値の増減
// -----------------------------------------------------------------------------
module.validates['capability'] = {
	actor: {
		notEmpty: [],
		regCheck: varRegCheck,
		varCheck: varVarCheck
	},
	capability: {
		notEmpty: [],
		list: [
			'0', '1', '2', '3', '4', '5', '6', '7',
			'maxhp', 'maxmp', 'atk', 'def', 'matk',
			'mdef', 'agi', 'luk'
		]
	},
	value: {
		notEmpty: [],
		regCheck: varRegCheck,
		varCheck: varVarCheck
	}
};
module.convertFuncs['capability'] = function(args) {
	var actor = {value: args['actor']};
	var params = {value: args['value']};
	var capability = this.convertList(args['capability'], {
		'0': 0,
		'1': 1,
		'2': 2,
		'3': 3,
		'4': 4,
		'5': 5,
		'6': 6,
		'7': 7,
		'maxhp': 0,
		'maxmp': 1,
		'atk': 2,
		'def': 3,
		'matk': 4,
		'mdef': 5,
		'agi': 6,
		'luk': 7
	});

	convertVarType(actor);
	convertVarType(params);
	convertVerOperate(params);

	return {
		code: 317,
		indent: this._indent,
		parameters: [
			actor.type,
			parseInt(actor.value, 10),
			capability,
			params.operate,
			params.type,
			parseInt(params.value, 10)
		]
	};
};
module.deconvertFuncs[317] = function(event) {
	var args = {};

	this.deconvertNumeric('actor', args, event.parameters[1]);
	deconvertVarType('actor', args, event.parameters[0]);
	this.deconvertNumeric('value', args, event.parameters[5]);
	deconvertVarOperate('value', args, event.parameters[3]);
	deconvertVarType('value', args, event.parameters[4]);
	this.deconvertList('capability', args, event.parameters[2], [
		0, 'maxhp',
		1, 'maxmp',
		2, 'atk',
		3, 'def',
		4, 'matk',
		5, 'mdef',
		6, 'agi',
		7, 'luk'
	]);

	return { name: 'capability', args: args };
};

// -----------------------------------------------------------------------------
// @skill/318:スキルの増減
// -----------------------------------------------------------------------------
module.validates['skill'] = {
	actor: {
		notEmpty: [],
		regCheck: varRegCheck,
		varCheck: varVarCheck
	},
	value: {
		notEmpty: [],
		isNumeric: [null, null],
		zeroCheck: []
	}
};
module.convertFuncs['skill'] = function(args) {
	var actor = {value: args['actor']};
	var params = {value: args['value']};

	convertVarType(actor);
	convertVerOperate(params);

	return {
		code: 318,
		indent: this._indent,
		parameters: [
			actor.type,
			parseInt(actor.value, 10),
			params.operate,
			parseInt(params.value, 10)
		]
	};
};
module.deconvertFuncs[318] = function(event) {
	var args = {};

	this.deconvertNumeric('actor', args, event.parameters[1]);
	deconvertVarType('actor', args, event.parameters[0]);
	this.deconvertNumeric('value', args, event.parameters[3]);
	deconvertVarOperate('value', args, event.parameters[2]);

	return { name: 'skill', args: args };
};

// -----------------------------------------------------------------------------
// @equip/319:装備の変更
// -----------------------------------------------------------------------------
module.validates['equip'] = {
	actor: {
		notEmpty: [],
		isNumeric: [1, null]
	},
	part: {
		notEmpty: [],
		list: [
			'1', '2', '3', '4', '5',
			'weapon', 'shield', 'head', 'body', 'accessory'
		]
	},
	id: {
		notEmpty: [],
		isNumeric: [0, null]
	}
};
module.convertFuncs['equip'] = function(args) {
	var part = this.convertList(args['part'], {
		'1': 1,
		'2': 2,
		'3': 3,
		'4': 4,
		'5': 5,
		'weapon': 1,
		'shield': 2,
		'head': 3,
		'body': 4,
		'accessory': 5
	});

	return {
		code: 319,
		indent: this._indent,
		parameters: [
			parseInt(args['actor'], 10),
			part,
			parseInt(args['id'], 10)
		]
	};
};
module.deconvertFuncs[319] = function(event) {
	var args = {};

	this.deconvertNumeric('actor', args, event.parameters[0]);
	this.deconvertList('part', args, event.parameters[1], [
		0, 'weapon',
		1, 'shield',
		2, 'head',
		3, 'body',
		4, 'accessory'
	]);
	this.deconvertNumeric('id', args, event.parameters[2]);

	return { name: 'equip', args: args };
};

// -----------------------------------------------------------------------------
// @name/320:名前の変更
// -----------------------------------------------------------------------------
module.validates['name'] = {
	actor: {
		notEmpty: [],
		isNumeric: [1, null]
	},
	value: {notEmpty: []}
};
module.convertFuncs['name'] = function(args) {
	return {
		code: 320,
		indent: this._indent,
		parameters: [
			parseInt(args['actor'], 10),
			args['value']
		]
	};
};
module.deconvertFuncs[320] = function(event) {
	var args = {};

	this.deconvertNumeric('actor', args, event.parameters[0]);
	this.deconvertString('value', args, event.parameters[1]);

	return { name: 'name', args: args };
};

// -----------------------------------------------------------------------------
// @class/321:職業の変更
// -----------------------------------------------------------------------------
module.validates['class'] = {
	actor: {
		notEmpty: [],
		isNumeric: [1, null]
	},
	value: {
		notEmpty: [],
		isNumeric: [1, null]
	},
	keepLevel: { isBool: [] }
};
module.convertFuncs['class'] = function(args) {
	return {
		code: 321,
		indent: this._indent,
		parameters: [
			parseInt(args['actor'], 10),
			parseInt(args['value'], 10),
			(args['keepLevel'] || 'false') == 'true' ? true : false
		]
	};
};
module.deconvertFuncs[321] = function(event) {
	var args = {};

	this.deconvertNumeric('actor', args, event.parameters[0]);
	this.deconvertNumeric('value', args, event.parameters[1]);
	this.deconvertBoolean('keepLevel', args, event.parameters[2], false);

	return { name: 'class', args: args };
};

// -----------------------------------------------------------------------------
// @nickname/324:二つ名の変更
// -----------------------------------------------------------------------------
module.validates['nickname'] = {
	actor: {
		notEmpty: [],
		isNumeric: [1, null]
	},
	value: {notEmpty: []}
};
module.convertFuncs['nickname'] = function(args) {
	return {
		code: 324,
		indent: this._indent,
		parameters: [
			parseInt(args['actor'], 10),
			args['value']
		]
	};
};
module.deconvertFuncs[324] = function(event) {
	var args = {};

	this.deconvertNumeric('actor', args, event.parameters[0]);
	this.deconvertString('value', args, event.parameters[1]);

	return { name: 'nickname', args: args };
};

// -----------------------------------------------------------------------------
// @profile/325:プロフィールの変更
// -----------------------------------------------------------------------------
module.validates['profile'] = {
	actor: {
		notEmpty: [],
		isNumeric: [1, null]
	},
	value: {}
};
module.convertFuncs['profile'] = function(args) {
	return {
		code: 325,
		indent: this._indent,
		parameters: [
			parseInt(args['actor'], 10),
			args['value']
		]
	};
};
module.splitFuncs['profile'] = function(text, tag) {
	var lines = [];
	this._index++;

	var args = this.getCommandArgs(text);

	while(!this.eoc()) {
		text = this.getLine();
		if (this.getCommandName(text) == 'profile_end') break;

		lines.push(text);

		this._index++;
	}

	if (this.eoc()) {
		throw new Tes.SyntaxError(
			'コメントタグが閉じられていません。',
			'syntax.profile.footer.notFound.error'
		);
	}

	this._data.push(
		this.createData.call(
			this,
			'profile', {
				actor: args['actor'],
				value: lines.join('\n')
			}
		)
	);
};
module.deconvertFuncs[325] = function(event) {
	var tags = [];
	var args = {};

	this.deconvertNumeric('actor', args, event.parameters[0]);
	this.deconvertString('value', args, event.parameters[1]);
	var lines = args.value.split('\n');
	delete args.value;

	tags.push({ name: 'profile', args: args });
	for (var i = 0; lines.length; i++) {
		tags.push(lines[i]);
	}
	tags.push({ name: 'profile_end'});

	return tags;
};

// -----------------------------------------------------------------------------
// @profile_end/---:プロフィールの終了
// -----------------------------------------------------------------------------
module.splitFuncs['profile_end'] = function(text, tag) {
	throw new Tes.SyntaxError(
		'プロフィールタグが開始されていません。',
		'syntax.profile.header.notFound.error'
	);
};

/*******************************************************************************
 * 移動
 ******************************************************************************/
// -----------------------------------------------------------------------------
// @map_move/201:場所移動
// -----------------------------------------------------------------------------
module.validates['map_move'] = {
	type: {list: ['const', 'var']},
	map: {
		notEmpty: [],
		isNumeric: [1, null]
	},
	x: {
		notEmpty: [],
		isNumeric: [0, null],
		varCheck2: ['type']
	},
	y: {
		notEmpty: [],
		isNumeric: [0, null],
		varCheck2: ['type']
	},
	direction: {
		list: [
			'0', '2', '4', '6', '8',
			'left', 'right', 'up', 'down'
		]
	},
	fade: {
		list: [
			'0', '1', '2',
			'black', 'white', 'none'
		]
	}
};
module.convertFuncs['map_move'] = function(args) {
	var type = this.convertList(args['type'], {
		'const': 0,
		'var': 1
	}, 0);
	var direction = this.convertList(args['direction'], {
		'2': 2,
		'4': 4,
		'6': 6,
		'8': 8,
		'down': 2,
		'left': 4,
		'right': 6,
		'up': 8
	}, 0);
	var fade = this.convertList(args['fade'], {
		'black': 0,
		'white': 1,
		'none': 2
	}, 0);

	return {
		code: 201,
		indent: this._indent,
		parameters: [
			(args['type'] || 'const') == 'const' ? 0 : 1,
			parseInt(args['map'], 10),
			parseInt(args['x'], 10),
			parseInt(args['y'], 10),
			direction,
			fade
		]
	};
};
module.deconvertFuncs[201] = function(event) {
	var args = {};

	this.deconvertList('type', args, event.parameters[0], [
		0, 'const',
		1, 'var'
	], 0);
	this.deconvertList('direction', args, event.parameters[4], [
		2, 'down',
		4, 'left',
		6, 'right',
		8, 'up'
	], 0);
	this.deconvertList('fade', args, event.parameters[5], [
		0, 'black',
		1, 'white',
		2, 'none'
	], 0);
	this.deconvertNumeric('map', args, event.parameters[1]);
	this.deconvertNumeric('x', args, event.parameters[2]);
	this.deconvertNumeric('y', args, event.parameters[3]);

	return { name: 'map_move', args: args };
};

// -----------------------------------------------------------------------------
// @vehicle_pos/202:乗り物の位置設定
// -----------------------------------------------------------------------------
module.validates['vehicle_pos'] = {
	vehicle: {
		notEmpty: [],
		list: [
			'0', '1', '2',
			'boat', 'ship', 'airship'
		]
	},
	type: {list: ['const', 'var']},
	map: {
		notEmpty: [],
		isNumeric: [1, null]
	},
	x: {
		notEmpty: [],
		isNumeric: [0, null],
		varCheck2: ['type']
	},
	y: {
		notEmpty: [],
		isNumeric: [0, null],
		varCheck2: ['type']
	}
};
module.convertFuncs['vehicle_pos'] = function(args) {
	var vehicle = this.convertList(args['vehicle'], {
		'0': 0,
		'1': 1,
		'2': 2,
		'boat': 0,
		'ship': 1,
		'airship': 2
	});
	var type = this.convertList(args['type'], {
		'const': 0,
		'var': 1
	}, 0);

	return {
		code: 202,
		indent: this._indent,
		parameters: [
			vehicle,
			type,
			parseInt(args['map'], 10),
			parseInt(args['x'], 10),
			parseInt(args['y'], 10)
		]
	};
};
module.deconvertFuncs[202] = function(event) {
	var args = {};

	this.deconvertList('vehicle', args, event.parameters[0], [
		0, 'boat',
		1, 'ship',
		2, 'airship'
	]);
	this.deconvertList('type', args, event.parameters[1], [
		0, 'const',
		1, 'var'
	]);
	this.deconvertNumeric('map', args, event.parameters[2]);
	this.deconvertNumeric('x', args, event.parameters[3]);
	this.deconvertNumeric('y', args, event.parameters[4]);

	return { name: 'vehicle_pos', args: args };
};

// -----------------------------------------------------------------------------
// @event_pos/203:イベントの位置設定
// -----------------------------------------------------------------------------
module.validates['event_pos'] = {
	id: {
		notEmpty: [],
		isNumeric: [-1, null]
	},
	type: {list: ['const', 'var', 'target']},
	x: {
		notEmpty: [],
		isNumeric: [0, null],
		varCheck2: ['type']
	},
	y: {
		notEmpty: [],
		isNumeric: [0, null],
		varCheck2: ['type']
	},
	direction: {
		list: [
			'0', '2', '4', '6', '8',
			'left', 'right', 'up', 'down'
		]
	}
};
module.convertFuncs['event_pos'] = function(args) {
	var type = this.convertList(args['type'], {
		'const': 0,
		'var': 1,
		'target': 2
	}, 0);
	var direction = this.convertList(args['direction'], {
		'2': 2,
		'4': 4,
		'6': 6,
		'8': 8,
		'down': 2,
		'left': 4,
		'right': 6,
		'up': 8
	}, 0);

	return {
		code: 203,
		indent: this._indent,
		parameters: [
			parseInt(args['id'], 10),
			type,
			parseInt(args['x'], 10),
			parseInt(args['y'], 10),
			direction
		]
	}
};
module.deconvertFuncs[203] = function(event) {
	var args = {};

	this.deconvertList('type', args, event.parameters[1], [
		0, 'const',
		1, 'var',
		2, 'target'
	]);
	this.deconvertList('direction', args, event.parameters[4], [
		2, 'down',
		4, 'left',
		6, 'right',
		8, 'up'
	], 0);
	this.deconvertNumeric('id', args, event.parameters[0]);
	this.deconvertNumeric('x', args, event.parameters[2]);
	this.deconvertNumeric('y', args, event.parameters[3]);

	return { name: 'event_pos', args: args };
};

// -----------------------------------------------------------------------------
// @scroll_map/204:マップのスクロール
// -----------------------------------------------------------------------------
module.validates['scroll_map'] = {
	direction: {
		notEmpty: [],
		list: [
			'2', '4', '6', '8',
			'left', 'right', 'up', 'down'
		]
	},
	num: {
		notEmpty: [],
		isNumeric: [1, 100]
	},
	speed: {isNumeric: [1, 6]}
};
module.convertFuncs['scroll_map'] = function(args) {
	var direction = this.convertList(args['direction'], {
		'2': 2,
		'4': 4,
		'6': 6,
		'8': 8,
		'down': 2,
		'left': 4,
		'right': 6,
		'up': 8
	});

	return {
		code: 204,
		indent: this._indent,
		parameters: [
			direction,
			parseInt(args['num'], 10),
			parseInt(args['speed'] || 4, 10)
		]
	}
};
module.deconvertFuncs[204] = function(event) {
	var args = {};

	this.deconvertList('direction', args, event.parameters[0], [
		2, 'down',
		4, 'left',
		6, 'right',
		8, 'up'
	]);
	this.deconvertNumeric('num', args, event.parameters[1]);
	this.deconvertNumeric('speed', args, event.parameters[2]);

	return { name: 'scroll_map', args: args };
};

// -----------------------------------------------------------------------------
// @route_h/205:移動ルートの設定(ヘッダ)
// -----------------------------------------------------------------------------
module.validates['route_h'] = {
	event: {
		notEmpty: [],
		isNumeric: [-1, null]
	},
	repeat: {isBool: []},
	skip: {isBool: []},
	wait: {isBool: []}
};
module.splitFuncs['route_h'] = function(text, tag) {
	// ヘッダはそのまま一般処理
	this.splitNormal.call(this, text, tag);

	// 移動ルート格納
	var routeCount = 0;
	while (!this.eoc()) {
		text = this.getNextLine();
		tag = this.getCommandName(text);

		if (tag != 'route') break;

		this._index++;
		routeCount++;
		splitRoute.call(this, text, tag);
	}

	if (routeCount == 0) {
		throw new Tes.SyntaxError(
			'移動ルートが設定されていません。',
			'syntax.route.empty.error'
		);
	}
};
module.convertFuncs['route_h'] = function(args) {
	var route = {
		repeat: (args['repeat'] || 'false') == 'true' ? true : false,
		skip: (args['skip'] || 'false') == 'true' ? true : false,
		wait: (args['wait'] || 'false') == 'true' ? true : false
	};

	// 同時true防止
	if (route.repeat && route.wait) route.repeat = false;

	return {
		code: 205,
		indent: this._indent,
		parameters: [
			parseInt(args['event'], 10),
			route
		]
	}
};
module.deconvertFuncs[205] = function(event) {
	var args = {};

	this.deconvertNumeric('event', args, event.parameters[0]);
	deconvertRoute('repert', 'skip', 'wait', args, event.parameters[1]);

	return { name: 'route_h', args: args };
};

// -----------------------------------------------------------------------------
// @route/505:移動ルートの設定
// -----------------------------------------------------------------------------
module.validates['route'] = {
	code: {
		notEmpty: []
	}
};
module.splitFuncs['route'] = function(text, tag) {
	throw new Tes.SyntaxError(
		'移動ルートのヘッダがありません。',
		'syntax.route.header.notFound.error'
	);
};
module.convertFuncs['route'] = function(args) {
	var code;
	var param = [];

	switch (args['code']) {
		case '1': case 'down':
			code = 1; break;
		case '2': case 'left':
			code = 2; break;
		case '3': case 'right':
			code = 3; break;
		case '4': case 'up':
			code = 4; break;
		case '5': case 'dl':
			code = 5; break;
		case '6': case 'dr':
			code = 6; break;
		case '7': case 'ul':
			code = 7; break;
		case '8': case 'ur':
			code = 8; break;
		case '9': case 'random':
			code = 9; break;
		case '10': case 'toward':
			code = 10; break;
		case '11': case 'away':
			code = 11; break;
		case '12': case 'foward':
			code = 12; break;
		case '13': case 'backward':
			code = 13; break;
		case '14': case 'jump':
			code = 14;
			param[0] = Tes.CommonFunctions.checkNumeric.call(this, 'x', args, -100, 100, Tes.ConvertError);
			param[1] = Tes.CommonFunctions.checkNumeric.call(this, 'y', args, -100, 100, Tes.ConvertError);

			break;
		case '15': case 'wait':
			code = 15;
			param[0] = Tes.CommonFunctions.checkNumeric.call(this, 'time', args, 1, 999, Tes.ConvertError) || 60;

			break;
		case '16': case 'turn_down':
			code = 16; break;
		case '17': case 'turn_left':
			code = 17; break;
		case '18': case 'turn_right':
			code = 18; break;
		case '19': case 'turn_up':
			code = 19; break;
		case '20': case 'turn_90_r':
			code = 20; break;
		case '21': case 'turn_90_l':
			code = 21; break;
		case '22': case 'turn_180':
			code = 22; break;
		case '23': case 'turn_90_rl': case 'turn_90_lr':
			code = 23; break;
		case '24': case 'turn_random':
			code = 24; break;
		case '25': case 'turn_toward':
			code = 25; break;
		case '26': case 'turn_away':
			code = 26; break;
		case '27': case 'switch_on': case 'sw_on':
			code = 27;
			Tes.CommonFunctions.checkNotEmpty.call(this, 'id', args, Tes.ConvertError);
			param[0] = Tes.CommonFunctions.checkNumeric.call(this, 'id', args, 1, null, Tes.ConvertError);

			break;
		case '28': case 'switch_off': case 'sw_off':
			Tes.CommonFunctions.checkNotEmpty.call(this, 'id', args, Tes.ConvertError);
			param[0] = Tes.CommonFunctions.checkNumeric.call(this, 'id', args, 1, null, Tes.ConvertError);

			code = 28; break;
		case '29': case 'change_speed':
			code = 29;
			param[0] = Tes.CommonFunctions.checkNumeric.call(this, 'speed', args, 1, 6, Tes.ConvertError) || 3;

			break;
		case '30': case 'change_freq':
			code = 30;
			param[0] = Tes.CommonFunctions.checkNumeric.call(this, 'freq', args, 1, 5, Tes.ConvertError) || 3;

			break;
		case '31': case 'walk_anime_on':
			code = 31; break;
		case '32': case 'walk_anime_off':
			code = 32; break;
		case '33': case 'step_anime_on':
			code = 33; break;
		case '34': case 'step_anime_off':
			code = 34; break;
		case '35': case 'dir_fix_on':
			code = 35; break;
		case '36': case 'dir_fix_off':
			code = 36; break;
		case '37': case 'through_on':
			code = 37; break;
		case '38': case 'through_off':
			code = 38; break;
		case '39': case 'transparent_on':
			code = 39; break;
		case '40': case 'transparent_off':
			code = 40; break;
		case '41': case 'change_graphic':
			code = 41;
			Tes.CommonFunctions.checkNotEmpty.call(this, 'name', args, Tes.ConvertError);
			Tes.CommonFunctions.checkNotEmpty.call(this, 'index', args, Tes.ConvertError);

			param[0] = args['name'];
			param[1] = Tes.CommonFunctions.checkNumeric.call(this, 'index', args, 0, 7, Tes.ConvertError);

			break;
		case '42': case 'change_opacity':
			code = 42;
			param[0] = Tes.CommonFunctions.checkNumeric.call(this, 'opacity', args, 0, 255, Tes.ConvertError) || 255;

			break;
		case '43': case 'change_blend':
			code = 43;
			Tes.CommonFunctions.checkList.call(
				this,
				'blend',
				args, [
					'0', '1', '2', '3',
					'normal', 'add', 'multiply', 'screen'
				]
			)
			param[0] = this.convertList(params[0], {
				'0': 0,
				'1': 1,
				'2': 2,
				'3': 3,
				'normal': 0,
				'add': 1,
				'multiply': 2,
				'screen': 3
			}, 0);

			break;
		case '44': case 'play_se':
			code = 44;
			param[0] = {
				name: args['name'] || '',
				volume: Tes.CommonFunctions.checkNumeric.call(this, 'volume', args, 0, 100, Tes.ConvertError) || 90,
				pitch: Tes.CommonFunctions.checkNumeric.call(this, 'pitch', args, 50, 150, Tes.ConvertError) || 100,
				pan: Tes.CommonFunctions.checkNumeric.call(this, 'pan', args, -100, 100, Tes.ConvertError) || 0
			};

			break;
		case '45': case 'script':
			code = 45;
			Tes.CommonFunctions.checkNotEmpty.call(this, 'script', args, Tes.ConvertError);
			param[0] = args['script'];

			break;
		default:
			throw new Tes.ConvertError(
				'存在しない移動コマンドです。',
				'syntax.route.command.notFound.error'
			);
	}

	return {
		code: 505,
		indent: this._indent,
		parameters: [
			{
				code: code,
				indent: null,
				param: param
			}
		]
	}
};
module.deconvertFuncs[505] = function(event) {
	var route = event.parameters[0];
	var args = {};

	if (route == null) {
		throw new Tes.DeconvertError(
			'ルートオブジェクトが未設定です。',
			'deconverter.route.undefined.error'
		);
	}

	this.deconvertList('code', args, route.code, [
		1, 'down',
		2, 'left',
		3, 'right',
		4, 'up',
		5, 'dl',
		6, 'dr',
		7, 'ul',
		8, 'ur',
		9, 'random',
		10, 'toward',
		11, 'away',
		12, 'foward',
		13, 'backward',
		14, 'jump',
		15, 'wait',
		16, 'turn_down',
		17, 'turn_left',
		18, 'turn_right',
		19, 'turn_up',
		20, 'turn_90_r',
		21, 'turn_90_l',
		22, 'turn_180',
		23, 'turn_90_rl',
		24, 'turn_random',
		25, 'turn_toward',
		26, 'turn_away',
		27, 'switch_on',
		28, 'switch_off',
		29, 'change_speed',
		30, 'change_freq',
		31, 'walk_anime_on',
		32, 'walk_anime_off',
		33, 'step_anime_on',
		34, 'step_anime_off',
		35, 'dir_fix_on',
		36, 'dir_fix_off',
		37, 'through_on',
		38, 'through_off',
		39, 'transparent_on',
		40, 'transparent_off',
		41, 'change_graphic',
		42, 'change_opacity',
		43, 'change_blend',
		44, 'play_se',
		45, 'script'
	]);

	switch (args.code) {
		case 'jump':
			this.deconvertNumeric('x', args, route.parameters[0]);
			this.deconvertNumeric('y', args, route.parameters[1]);

			break;
		case 'wait':
			this.deconvertNumeric('time', args, route.parameters[0]);

			break;
		case 'switch_on':
		case 'switch_off':
			this.deconvertNumeric('id', args, route.parameters[0]);

			break;
		case 'change_speed':
			this.deconvertNumeric('speed', args, route.parameters[0]);

			break;
		case 'change_freq':
			this.deconvertNumeric('freq', args, route.parameters[0]);

			break;
		case 'change_graphic':
			this.deconvertString('name', args, route.parameters[0]);
			this.deconvertNumeric('index', args, route.parameters[1]);

			break;
		case 'change_opacity':
			this.deconvertNumeric('opacity', args, route.parameters[0]);

			break;
		case 'change_blend':
			this.deconvertList('blend', args, route.parameters[0], [
				0, 'normal',
				1, 'add',
				2, 'multiply',
				3, 'screen'
			]);

			break;
		case 'play_se':
			this.deconvertAudio('name', 'volume', 'pitch', 'pan', args, route.parameters[0], '');

			break;
		case 'script':
			this.deconvertString('script', args, route.parameters[0]);

			break;
	}

	return { name: 'route', args: args }
};

// -----------------------------------------------------------------------------
// @vehicle/206:乗り物の乗降
// -----------------------------------------------------------------------------
module.convertFuncs['vehicle'] = function(args) {
	return {
		code: 206,
		indent: this._indent,
		parameters: []
	}
};
module.deconvertFuncs[206] = function(event) {
	return { name: 'vehicle' };
};

/**
 * 移動ルートの分割処理
 */
function splitRoute(text, tag) {
	return this.splitNormal.call(this, text, tag);
}

/*******************************************************************************
 * キャラクター
 ******************************************************************************/
// -----------------------------------------------------------------------------
// @transparent/211:透明状態の変更
// -----------------------------------------------------------------------------
module.validates['transparent'] = {
	flag: {
		notEmpty: [],
		list: flagList
	}
};
module.convertFuncs['transparent'] = function(args) {
	return {
		code: 211,
		indent: this._indent,
		parameters: [
			this.convertFlag(args['flag'])
		]
	}
};
module.deconvertFuncs[211] = function(event) {
	var args = {};

	this.deconvertFlag('flag', args, event.parameters[0]);

	return { name: 'transparent', args: args };
};

// -----------------------------------------------------------------------------
// @followers/216:隊列歩行の変更
// -----------------------------------------------------------------------------
module.validates['followers'] = {
	flag: {
		notEmpty: [],
		list: flagList
	}
};
module.convertFuncs['followers'] = function(args) {
	return {
		code: 216,
		indent: this._indent,
		parameters: [
			this.convertFlag(args['flag'])
		]
	}
};
module.deconvertFuncs[216] = function(event) {
	var args = {};

	this.deconvertFlag('flag', args, event.parameters[0]);

	return { name: 'followers', args: args };
};

// -----------------------------------------------------------------------------
// @gather/217:隊列メンバーの集合
// -----------------------------------------------------------------------------
module.convertFuncs['gather'] = function(args) {
	return {
		code: 217,
		indent: this._indent,
		parameters: []
	}
};
module.deconvertFuncs[217] = function(event) {
	return { name: 'gather' };
};

// -----------------------------------------------------------------------------
// @anime/212:アニメーションの表示
// -----------------------------------------------------------------------------
module.validates['anime'] = {
	target: {
		notEmpty: [],
		isNumeric: [-1, null]
	},
	anime: {
		notEmpty: [],
		isNumeric: [1, null]
	},
	wait: {isBool: []}
};
module.convertFuncs['anime'] = function(args) {
	return {
		code: 212,
		indent: this._indent,
		parameters: [
			parseInt(args['target'], 10),
			parseInt(args['anime'], 10),
			(args['wait'] || false) == 'true' ? true : false
		]
	}
};
module.deconvertFuncs[212] = function(event) {
	var args = {};

	this.deconvertNumeric('target', args, event.parameters[0]);
	this.deconvertNumeric('anime', args, event.parameters[1]);
	this.deconvertBoolean('wait', args, event.parameters[2]);

	return { name: 'anime', args: args };
};

// -----------------------------------------------------------------------------
// @balloon/213:フキダシアイコンの表示
// -----------------------------------------------------------------------------
module.validates['balloon'] = {
	target: {
		notEmpty: [],
		isNumeric: [-1, null]
	},
	balloon: {
		notEmpty: [],
		isNumeric: [1, null]
	},
	wait: {isBool: []}
};
module.convertFuncs['balloon'] = function(args) {
	return {
		code: 213,
		indent: this._indent,
		parameters: [
			parseInt(args['target'], 10),
			parseInt(args['balloon'], 10),
			(args['wait'] || false) == 'true' ? true : false
		]
	}
};
module.deconvertFuncs[213] = function(event) {
	var args = {};

	this.deconvertNumeric('target', args, event.parameters[0]);
	this.deconvertNumeric('balloon', args, event.parameters[1]);
	this.deconvertBoolean('wait', args, event.parameters[2]);

	return { name: 'balloon', args: args };
};

// -----------------------------------------------------------------------------
// @erace/214:イベントの一時消去
// -----------------------------------------------------------------------------
module.convertFuncs['erace'] = function(args) {
	return {
		code: 214,
		indent: this._indent,
		parameters: []
	}
};
module.deconvertFuncs[214] = function(event) {
	return { name: 'erace' };
};

/*******************************************************************************
 * ピクチャ
 ******************************************************************************/
// -----------------------------------------------------------------------------
// @picture/231:ピクチャの表示
// -----------------------------------------------------------------------------
module.validates['picture'] = {
	layer: {
		notEmpty: [],
		isNumeric: [1, 100]
	},
	name: {notEmpty: []},
	origin: {
		list: [
			'0', '1',
			'ul', 'center'
		]
	},
	type: {
		list: [
			'0', '1',
			'const', 'var'
		]
	},
	x: {
		notEmpty: [],
		isNumeric: [-9999, 9999],
		varCheck2: ['type']
	},
	y: {
		notEmpty: [],
		isNumeric: [-9999, 9999],
		varCheck2: ['type']
	},
	zoom_x: {isNumeric: [0, 2000]},
	zoom_y: {isNumeric: [0, 2000]},
	transparent: {isNumeric: [0, 255]},
	blend: {
		list: [
			'0', '1', '2', '3',
			'normal', 'add', 'multiply', 'screen'
		]
	}
};
module.convertFuncs['picture'] = function(args) {
	var origin = this.convertList(args['origin'], {
		'0': 0,
		'1': 1,
		'ul': 0,
		'center': 1
	}, 0);
	var type = this.convertList(args['type'], {
		'0': 0,
		'1': 1,
		'const': 0,
		'var': 1
	}, 0);
	var blend = this.convertList(args['blend'], {
		'0': 0,
		'1': 1,
		'2': 2,
		'3': 3,
		'normal': 0,
		'add': 1,
		'multiply': 2,
		'screen': 3
	}, 0);

	return {
		code: 231,
		indent: this._indent,
		parameters: [
			parseInt(args['layer'], 10),
			args['name'],
			origin,
			type,
			parseInt(args['x'], 10),
			parseInt(args['y'], 10),
			parseInt(args['zoom_x'] || 100, 10),
			parseInt(args['zoom_y'] || 100, 10),
			parseInt(args['transparent'] || 255, 10),
			blend
		]
	}
};
module.deconvertFuncs[231] = function(event) {
	var args = {};

	this.deconvertList('origin', args, event.parameters[2], [
		0, 'ul',
		1, 'center'
	]);
	this.deconvertList('type', args, event.parameters[3], [
		0, 'const',
		1, 'var'
	]);
	this.deconvertNumeric('layer', args, event.parameters[0]);
	this.deconvertString('name', args, event.parameters[1]);
	this.deconvertNumeric('x', args, event.parameters[4]);
	this.deconvertNumeric('y', args, event.parameters[5]);
	this.deconvertNumeric('zoom_x', args, event.parameters[6]);
	this.deconvertNumeric('zoom_y', args, event.parameters[7]);
	this.deconvertNumeric('transparent', args, event.parameters[8]);
	this.deconvertList('blend', args, event.parameters[9], [
		0, 'normal',
		1, 'add',
		2, 'multiply',
		3, 'screen'
	]);

	return { name: 'picture', args: args };
};

// -----------------------------------------------------------------------------
// @picture_move/232:ピクチャの移動
// -----------------------------------------------------------------------------
module.validates['picture_move'] = {
	layer: {
		notEmpty: [],
		isNumeric: [1, 100]
	},
	origin: {
		list: [
			'0', '1',
			'ul', 'center'
		]
	},
	type: {
		list: [
			'0', '1',
			'const', 'var'
		]
	},
	x: {
		notEmpty: [],
		isNumeric: [-9999, 9999],
		varCheck2: ['type']
	},
	y: {
		notEmpty: [],
		isNumeric: [-9999, 9999],
		varCheck2: ['type']
	},
	zoom_x: {isNumeric: [0, 2000]},
	zoom_y: {isNumeric: [0, 2000]},
	transparent: {isNumeric: [0, 255]},
	blend: {
		list: [
			'0', '1', '2', '3',
			'normal', 'add', 'multiply', 'screen'
		]
	},
	time: {isNumeric: [1, 600]},
	wait: {isBool: []}
};
module.convertFuncs['picture_move'] = function(args) {
	var origin = this.convertList(args['origin'], {
		'0': 0,
		'1': 1,
		'ul': 0,
		'center': 1
	}, 0);
	var type = this.convertList(args['type'], {
		'0': 0,
		'1': 1,
		'const': 0,
		'var': 1
	}, 0);
	var blend = this.convertList(args['blend'], {
		'0': 0,
		'1': 1,
		'2': 2,
		'3': 3,
		'normal': 0,
		'add': 1,
		'multiply': 2,
		'screen': 3
	}, 0);

	return {
		code: 232,
		indent: this._indent,
		parameters: [
			parseInt(args['layer'], 10),
			null,
			origin,
			type,
			parseInt(args['x'], 10),
			parseInt(args['y'], 10),
			parseInt(args['zoom_x'] || 100, 10),
			parseInt(args['zoom_y'] || 100, 10),
			parseInt(args['transparent'] || 255, 10),
			blend,
			parseInt(args['time'] || 60, 10),
			(args['wait'] || 'true') == 'true' ? true : false
		]
	}
};
module.deconvertFuncs[232] = function(event) {
	var args = {};

	this.deconvertList('origin', args, event.parameters[2], [
		0, 'ul',
		1, 'center'
	]);
	this.deconvertList('type', args, event.parameters[3], [
		0, 'const',
		1, 'var'
	]);
	this.deconvertNumeric('layer', args, event.parameters[0]);
	this.deconvertNumeric('x', args, event.parameters[4]);
	this.deconvertNumeric('y', args, event.parameters[5]);
	this.deconvertNumeric('zoom_x', args, event.parameters[6]);
	this.deconvertNumeric('zoom_y', args, event.parameters[7]);
	this.deconvertNumeric('transparent', args, event.parameters[8]);
	this.deconvertList('blend', args, event.parameters[9], [
		0, 'normal',
		1, 'add',
		2, 'multiply',
		3, 'screen'
	]);
	this.deconvertNumeric('time', args, event.parameters[10]);
	this.deconvertBoolean('wait', args, event.parameters[11]);

	return { name: 'picture_move', args: args };
};

// -----------------------------------------------------------------------------
// @picture_rotation/233:ピクチャの回転
// -----------------------------------------------------------------------------
module.validates['picture_rotation'] = {
	layer: {
		notEmpty: [],
		isNumeric: [1, 100]
	},
	speed: {isNumeric: [-90, 90]}
};
module.convertFuncs['picture_rotation'] = function(args) {
	return {
		code: 233,
		indent: this._indent,
		parameters: [
			parseInt(args['layer'], 10),
			parseInt(args['speed'] || 5, 10)
		]
	}
};
module.deconvertFuncs[233] = function(event) {
	var args = {};

	this.deconvertNumeric('layer', args, event.parameters[0]);
	this.deconvertNumeric('speed', args, event.parameters[1]);

	return { name: 'picture_rotation', args: args }
};

// -----------------------------------------------------------------------------
// @picture_tone/234:ピクチャの色調変更
// -----------------------------------------------------------------------------
module.validates['picture_tone'] = {
	layer: {
		notEmpty: [],
		isNumeric: [1, 100]
	},
	red: {isNumeric: [-255, 255]},
	blue: {isNumeric: [-255, 255]},
	green: {isNumeric: [-255, 255]},
	gray: {isNumeric: [0, 255]},
	time: {isNumeric: [0, 600]},
	wait: {isBool: []}
};
module.convertFuncs['picture_tone'] = function(args) {
	return {
		code: 234,
		indent: this._indent,
		parameters: [
			parseInt(args['layer'], 10),
			[
				parseFloat(args['red'] || 0.0),
				parseFloat(args['green'] || 0.0),
				parseFloat(args['blue'] || 0.0),
				parseFloat(args['gray'] || 0.0)
			],
			parseInt(args['time'] || 60, 10),
			(args['wait'] || 'true') == 'true' ? true : false
		]
	}
};
module.deconvertFuncs[234] = function(event) {
	var args = {};

	this.deconvertNumeric('layer', args, event.parameters[0]);
	this.deconvertColor('red', 'green', 'blue', 'gray', args, event.parameters[1]);
	this.deconvertNumeric('time', args, event.parameters[2]);
	this.deconvertNumeric('wait', args, event.parameters[3]);

	return { name: 'picture_tone', args: args };
};

// -----------------------------------------------------------------------------
// @picture_erace/235:ピクチャの消去
// -----------------------------------------------------------------------------
module.validates['picture_erace'] = {
	layer: {
		notEmpty: [],
		isNumeric: [1, 100]
	}
};
module.convertFuncs['picture_erace'] = function(args) {
	return {
		code: 235,
		indent: this._indent,
		parameters: [
			parseInt(args['layer'], 10)
		]
	}
};
module.deconvertFuncs[235] = function(event) {
	var args = {};

	this.deconvertNumeric('layer', args, event.parameters[0]);

	return { name: 'picture_erace', args: args };
};

/*******************************************************************************
 * タイミング
 ******************************************************************************/
// -----------------------------------------------------------------------------
// @wait/230:ウェイト
// -----------------------------------------------------------------------------
module.validates['wait'] = {
	time: {isNumeric: [1, 999]}
};
module.convertFuncs['wait'] = function(args) {
	return {
		code: 230,
		indent: this._indent,
		parameters: [parseInt(args['time'] || 60, 10)]
	}
};
module.deconvertFuncs[230] = function(event) {
	var args = {};

	this.deconvertNumeric('time', args, event.parameters[0]);

	return { name: 'wait', args: args };
};

/*******************************************************************************
 * 画面
 ******************************************************************************/
// -----------------------------------------------------------------------------
// @fadeout/221:画面のフェードアウト
// -----------------------------------------------------------------------------
module.convertFuncs['fadeout'] = function(args) {
	return {
		code: 221,
		indent: this._indent,
		parameters: []
	}
};
module.deconvertFuncs[221] = function(event) {
	return { name: 'fadeout' };
};

// -----------------------------------------------------------------------------
// @fadein/222:画面のフェードイン
// -----------------------------------------------------------------------------
module.convertFuncs['fadein'] = function(args) {
	return {
		code: 222,
		indent: this._indent,
		parameters: []
	}
};
module.deconvertFuncs[222] = function(event) {
	return { name: 'fadein' };
};

// -----------------------------------------------------------------------------
// @tone/223:画面の色調変更
// -----------------------------------------------------------------------------
module.validates['tone'] = {
	red: {isNumeric: [-255, 255]},
	blue: {isNumeric: [-255, 255]},
	green: {isNumeric: [-255, 255]},
	gray: {isNumeric: [0, 255]},
	time: {isNumeric: [1, 600]},
	wait: {isBool: []}
};
module.convertFuncs['tone'] = function(args) {
	return {
		code: 223,
		indent: this._indent,
		parameters: [
			[
				parseFloat(args['red'] || 0.0),
				parseFloat(args['green'] || 0.0),
				parseFloat(args['blue'] || 0.0),
				parseFloat(args['gray'] || 0.0)
			],
			parseInt(args['time'] || 60, 10),
			(args['wait'] || 'true') == 'true' ? true : false
		]
	}
};
module.deconvertFuncs[223] = function(event) {
	var args = {};

	this.deconvertColor('red', 'green', 'blue', 'gray', args, event.parameters[0]);
	this.deconvertNumeric('time', args, event.parameters[1]);
	this.deconvertBoolean('wait', args, event.parameters[2]);

	return { name: 'tone', args: args };
};

// -----------------------------------------------------------------------------
// @flash/224:画面のフラッシュ
// -----------------------------------------------------------------------------
module.validates['flash'] = {
	red: {isNumeric: [0, 255]},
	blue: {isNumeric: [0, 255]},
	green: {isNumeric: [0, 255]},
	strength: {isNumeric: [0, 255]},
	time: {isNumeric: [1, 600]},
	wait: {isBool: []}
};
module.convertFuncs['flash'] = function(args) {
	return {
		code: 224,
		indent: this._indent,
		parameters: [
			[
				parseFloat(args['red'] || 255.0),
				parseFloat(args['green'] || 255.0),
				parseFloat(args['blue'] || 255.0),
				parseFloat(args['strength'] || 255.0)
			],
			parseInt(args['time'] || 60, 10),
			(args['wait'] || 'true') == 'true' ? true : false
		]
	}
};
module.deconvertFuncs[224] = function(event) {
	var args = {};

	this.deconvertTone('red', 'green', 'blue', 'strength', args, event.parameters[0]);
	this.deconvertNumeric('time', args, event.parameters[1]);
	this.deconvertBoolean('wait', args, event.parameters[2]);

	return { name: 'flash', args: args };
};

// -----------------------------------------------------------------------------
// @shake/225:画面のシェイク
// -----------------------------------------------------------------------------
module.validates['shake'] = {
	strength: {isNumeric: [1, 9]},
	speed: {isNumeric: [1, 9]},
	time: {isNumeric: [1, 600]},
	wait: {isBool: []}
};
module.convertFuncs['shake'] = function(args) {
	return {
		code: 225,
		indent: this._indent,
		parameters: [
			parseInt(args['strength'] || 5, 10),
			parseInt(args['speed'] || 5, 10),
			parseInt(args['time'] || 60, 10),
			(args['wait'] || 'true') == 'true' ? true : false
		]
	}
};
module.deconvertFuncs[225] = function(event) {
	var args = {};

	this.deconvertNumeric('strength', args, event.parameters[0]);
	this.deconvertNumeric('speed', args, event.parameters[1]);
	this.deconvertNumeric('time', args, event.parameters[2]);
	this.deconvertBoolean('wait', args, event.parameters[3]);

	return { name: 'shake', args: args };
};

// -----------------------------------------------------------------------------
// @weather/236:天候の設定
// -----------------------------------------------------------------------------
module.validates['weather'] = {
	weather: {
		list: [
			'0', '1', '2', '3',
			'none', 'rain', 'storm', 'snow'
		]
	},
	strength: {isNumeric: [1, 9]},
	time: {isNumeric: [0, 600]},
	wait: {isBool: []}
};
module.convertFuncs['weather'] = function(args) {
	var weather = this.convertList(args['weather'], {
		'0': 'none',
		'1': 'rain',
		'2': 'storm',
		'3': 'snow',
		'none': 'none',
		'rain': 'rain',
		'storm': 'storm',
		'snow': 'snow'
	}, null);

	return {
		code: 236,
		indent: this._indent,
		parameters: [
			weather,
			parseInt(args['speed'] || 5, 10),
			parseInt(args['time'] || 60, 10),
			(args['wait'] || 'true') == 'true' ? true : false
		]
	}
};
module.deconvertFuncs[236] = function(event) {
	var args = {};

	this.deconvertString('weather', args, event.parameters[0], null);
	this.deconvertNumeric('speed', args, event.parameters[1]);
	this.deconvertNumeric('time', args, event.parameters[2]);
	this.deconvertBoolean('wait', args, event.parameters[3]);

	return { name: 'weather', args: args };
};

/*******************************************************************************
 * オーディオ・ビデオ
 ******************************************************************************/
// -----------------------------------------------------------------------------
// @bgm/241:BGMの演奏
// -----------------------------------------------------------------------------
module.validates['bgm'] = {
	volume: {isNumeric: [0, 100]},
	pitch: {isNumeric: [50, 150]},
	pan: {isNumeric: [-100, 100]}
};
module.convertFuncs['bgm'] = function(args) {
	return {
		code: 241,
		indent: this._indent,
		parameters: [
			{
				name: args['name'] || '',
				volume: parseInt(args['volume'] || 90, 10),
				pitch: parseInt(args['pitch'] || 100, 10),
				pan: parseInt(args['pan'] || 0, 10)
			}
		]
	}
};
module.deconvertFuncs[241] = function(event) {
	var args = {};

	this.deconvertAudio('name', 'volume', 'pitch', 'pan', args, event.parameters[0], '');

	return { name: 'bgm', args: args };
};

// -----------------------------------------------------------------------------
// @fadeout_bgm/242:BGMのフェードアウト
// -----------------------------------------------------------------------------
module.validates['fadeout_bgm'] = {
	time: {isNumeric: [1, 60]}
};
module.convertFuncs['fadeout_bgm'] = function(args) {
	return {
		code: 242,
		indent: this._indent,
		parameters: [
			parseInt(args['time'] || 10, 10)
		]
	}
};
module.deconvertFuncs[242] = function(event) {
	var args = {};
	
	this.deconvertNumeric('time', args, event.parameters[0]);

	return { name: 'fadeout_bgm', args: args };
};

// -----------------------------------------------------------------------------
// @save_bgm/243:BGMの保存
// -----------------------------------------------------------------------------
module.convertFuncs['save_bgm'] = function(args) {
	return {
		code: 243,
		indent: this._indent,
		parameters: []
	}
};
module.deconvertFuncs[243] = function(event) {
	return { name: 'save_bgm' };
};

// -----------------------------------------------------------------------------
// @resume_bgm/244:BGMの再開
// -----------------------------------------------------------------------------
module.convertFuncs['resume_bgm'] = function(args) {
	return {
		code: 244,
		indent: this._indent,
		parameters: []
	}
};
module.deconvertFuncs[244] = function(event) {
	return { name: 'resume_bgm' };
};

// -----------------------------------------------------------------------------
// @bgs/245:BGSの演奏
// -----------------------------------------------------------------------------
module.validates['bgs'] = {
	volume: {isNumeric: [0, 100]},
	pitch: {isNumeric: [50, 150]},
	pan: {isNumeric: [-100, 100]}
};
module.convertFuncs['bgs'] = function(args) {
	return {
		code: 245,
		indent: this._indent,
		parameters: [
			{
				name: args['name'] || '',
				volume: parseInt(args['volume'] || 90, 10),
				pitch: parseInt(args['pitch'] || 100, 10),
				pan: parseInt(args['pan'] || 0, 10)
			}
		]
	}
};
module.deconvertFuncs[245] = function(event) {
	var args = {};

	this.deconvertAudio('name', 'volume', 'pitch', 'pan', args, event.parameters[0], '');

	return { name: 'bgs', args: args };
};

// -----------------------------------------------------------------------------
// @fadeout_bgs/246:BGSのフェードアウト
// -----------------------------------------------------------------------------
module.validates['fadeout_bgs'] = {
	time: {isNumeric: [1, 60]}
};
module.convertFuncs['fadeout_bgs'] = function(args) {
	return {
		code: 246,
		indent: this._indent,
		parameters: [
			parseInt(args['time'] || 10, 10)
		]
	}
};
module.deconvertFuncs[246] = function(event) {
	var args = {};

	this.deconvertNumeric('time', args, event.parameters[0]);

	return { name: 'fadeout_bgs', args: args };
};

// -----------------------------------------------------------------------------
// @me/249:MEの演奏
// -----------------------------------------------------------------------------
module.validates['me'] = {
	volume: {isNumeric: [0, 100]},
	pitch: {isNumeric: [50, 150]},
	pan: {isNumeric: [-100, 100]}
};
module.convertFuncs['me'] = function(args) {
	return {
		code: 249,
		indent: this._indent,
		parameters: [
			{
				name: args['name'] || '',
				volume: parseInt(args['volume'] || 90, 10),
				pitch: parseInt(args['pitch'] || 100, 10),
				pan: parseInt(args['pan'] || 0, 10)
			}
		]
	}
};
module.deconvertFuncs[249] = function(event) {
	var args = {};

	this.deconvertAudio('name', 'volume', 'pitch', 'pan', args, event.parameters[0], '');

	return { name: 'me', args: args };
};

// -----------------------------------------------------------------------------
// @se/250:SEの演奏
// -----------------------------------------------------------------------------
module.validates['se'] = {
	volume: {isNumeric: [0, 100]},
	pitch: {isNumeric: [50, 150]},
	pan: {isNumeric: [-100, 100]}
};
module.convertFuncs['se'] = function(args) {
	return {
		code: 250,
		indent: this._indent,
		parameters: [
			{
				name: args['name'] || '',
				volume: parseInt(args['volume'] || 90, 10),
				pitch: parseInt(args['pitch'] || 100, 10),
				pan: parseInt(args['pan'] || 0, 10)
			}
		]
	}
};
module.deconvertFuncs[250] = function(event) {
	var args = {};

	this.deconvertAudio('name', 'volume', 'pitch', 'pan', args, event.parameters[0], '');

	return { name: 'se', args: args };
};

// -----------------------------------------------------------------------------
// @stop_se/251:SEの停止
// -----------------------------------------------------------------------------
module.convertFuncs['stop_se'] = function(args) {
	return {
		code: 251,
		indent: this._indent,
		parameters: []
	}
};
module.deconvertFuncs[251] = function(event) {
	return { name: 'stop_se' };
};

// -----------------------------------------------------------------------------
// @movie/261:ムービーの再生
// -----------------------------------------------------------------------------
module.validates['movie'] = {
	name: {
		notEmpty: []
	}
};
module.convertFuncs['movie'] = function(args) {
	return {
		code: 261,
		indent: this._indent,
		parameters: [args['name']]
	}
};
module.deconvertFuncs[261] = function(event) {
	var args = {};

	this.deconvertString('name', args, event.parameters[0]);

	return { name: 'movie', args: args };
};

/*******************************************************************************
 * シーン制御
 ******************************************************************************/
// -----------------------------------------------------------------------------
// @battle/301:戦闘の処理
// -----------------------------------------------------------------------------
module.validates['battle'] = {
	id: {
		notEmpty: [],
		regCheck: varRegCheck,
		varCheck: varVarCheck
	},
	can_escape: {isBool: []},
	loss_continue: {isBool: []}
};
module.convertFuncs['battle'] = function(args) {
	var params = {value: args['id']};

	convertVarType(params);

	var id = parseInt(params.value, 10);
	var type = params.type;

	if (id == 0) type = 2;

	return {
		code: 301,
		indent: this._indent,
		parameters: [
			type,
			id,
			(args['can_escape'] || 'false') == 'true' ? true : false,
			(args['loss_continue'] || 'false') == 'true' ? true : false
		]
	}
};
module.deconvertFuncs[301] = function(event) {
	var args = {};

	if (event.parameters[0] == 2) {
		args.id = 0;
	} else {
		this.deconvertNumeric('id', args, event.parameters[1]);
		deconvertVarType('id', args, event.parameters[0]);
	}
	this.deconvertBoolean('can_escape', args, event.parameters[2]);
	this.deconvertBoolean('loss_continue', args, event.parameters[3]);

	return { name: 'battle', args: args };
};

// -----------------------------------------------------------------------------
// @battle_win/601:戦闘の処理(勝った場合)
// -----------------------------------------------------------------------------
module.splitFuncs['battle_win'] = function(text, tag) {
	var last = this._data[this._data.length - 1];
	if (this._data.length == 0 || (last.tag != 'battle' && last.tag != 'end')) {
		throw new Tas.SyntaxError(
			'戦闘分岐前にヘッダまたは終端タグがありません。',
			'syntax.battle.tag.notFound.error'
		);
	}

	this.splitNormal.call(this, text, tag);
};
module.convertFuncs['battle_win'] = function(args) {
	this._indent++;

	return {
		code: 601,
		indent: this._indent - 1,
		parameters: []
	}
};
module.deconvertFuncs[601] = function(event) {
	return { name: 'battle_win' };
};

// -----------------------------------------------------------------------------
// @battle_escape/602:戦闘の処理(逃げた場合)
// -----------------------------------------------------------------------------
module.splitFuncs['battle_escape'] = function(text, tag) {
	module.splitFuncs['battle_win'].call(this, text, tag);
};
module.convertFuncs['battle_escape'] = function(args) {
	this._indent++;

	return {
		code: 602,
		indent: this._indent - 1,
		parameters: []
	}
};
module.deconvertFuncs[602] = function(event) {
	return { name: 'battle_escape' };
};

// -----------------------------------------------------------------------------
// @battle_loss/603:戦闘の処理(負けた場合)
// -----------------------------------------------------------------------------
module.splitFuncs['battle_loss'] = function(text, tag) {
	module.splitFuncs['battle_win'].call(this, text, tag);
};
module.convertFuncs['battle_loss'] = function(args) {
	this._indent++;

	return {
		code: 603,
		indent: this._indent - 1,
		parameters: []
	}
};
module.deconvertFuncs[603] = function(event) {
	return { name: 'battle_loss' };
};

// -----------------------------------------------------------------------------
// @battle_end/604:戦闘の処理(分岐終了)
// -----------------------------------------------------------------------------
module.convertFuncs['battle_end'] = function(args) {
	return {
		code: 604,
		indent: this._indent,
		parameters: []
	}
};
module.deconvertFuncs[604] = function(event) {
	return { name: 'battle_end' };
};

// -----------------------------------------------------------------------------
// @shop/605:ショップの処理
// -----------------------------------------------------------------------------
module.validates['shop'] = {
	type: {
		notEmpty: [],
		list: [
			'0', '1', '2',
			'item', 'weapon', 'armor'
		]
	},
	id: {
		notEmpty: [],
		isNumeric: [1, null]
	},
	price: {
		list: [
			'0', '1',
			'db', 'value'
		]
	},
	value: {isNumeric: [0, 9999999]},
	buy_only: {isBool: []}
};
module.convertFuncs['shop'] = function(args) {
	var type = this.convertList(args['type'], {
		'0': 0,
		'1': 1,
		'2': 2,
		'item': 0,
		'weapon': 1,
		'armor': 2
	});
	var price = this.convertList(args['price'], {
		'0': 0,
		'1': 1,
		'db': 0,
		'value': 1
	}, 0)

	return {
		code: 605,
		indent: this._indent,
		parameters: [
			type,
			parseInt(args['id'], 10),
			price,
			parseInt(args['value'] || 50, 10),
			(args['buy_only'] || 'false') == 'true' ? true : false
		]
	}
};
module.deconvertFuncs[605] = function(event) {
	var args = {};

	this.deconvertList('type', args, event.parameters[0], [
		0, 'item',
		1, 'weapon',
		2, 'armor'
	]);
	this.deconvertList('price', args, event.parameters[2], [
		0, 'db',
		1, 'value'
	]);
	this.deconvertNumeric('id', args, event.parameters[1]);
	this.deconvertNumeric('value', args, event.parameters[3]);
	this.deconvertBoolean('buy_only', args, event.parameters[4], null);

	return { name: 'shop', args: args };
};
module.deconvertFuncs[302] = module.deconvertFuncs[605];

// -----------------------------------------------------------------------------
// @input_name/303:名前入力の処理
// -----------------------------------------------------------------------------
module.validates['input_name'] = {
	actor: {
		notEmpty: [],
		isNumeric: [1, null]
	},
	number: {
		isNumeric: [1, 16]
	}
};
module.convertFuncs['input_name'] = function(args) {
	return {
		code: 303,
		indent: this._indent,
		parameters: [
			parseInt(args['actor'], 10),
			parseInt(args['number'] || 6, 10)
		]
	}
};
module.deconvertFuncs[303] = function(event) {
	var args = {};

	this.deconvertNumeric('actor', args, event.parameters[0]);
	this.deconvertNumeric('number', args, event.parameters[1]);

	return { name: 'input_name', args: args };
};

// -----------------------------------------------------------------------------
// @menu_open/351:メニュー画面を開く
// -----------------------------------------------------------------------------
module.convertFuncs['menu_open'] = function(args) {
	return {
		code: 351,
		indent: this._indent,
		parameters: []
	}
};
module.deconvertFuncs[351] = function(event) {
	return { name: 'menu_open' };
};

// -----------------------------------------------------------------------------
// @save_open/352:セーブ画面を開く
// -----------------------------------------------------------------------------
module.convertFuncs['save_open'] = function(args) {
	return {
		code: 352,
		indent: this._indent,
		parameters: []
	}
};
module.deconvertFuncs[352] = function(event) {
	return { name: 'save_open' };
};

// -----------------------------------------------------------------------------
// @gameover/353:ゲームオーバー
// -----------------------------------------------------------------------------
module.convertFuncs['gameover'] = function(args) {
	return {
		code: 353,
		indent: this._indent,
		parameters: []
	}
};
module.deconvertFuncs[353] = function(event) {
	return { name: 'gameover' };
};

// -----------------------------------------------------------------------------
// @title_return/354:タイトル画面に戻す
// -----------------------------------------------------------------------------
module.convertFuncs['title_return'] = function(args) {
	return {
		code: 354,
		indent: this._indent,
		parameters: []
	}
};
module.deconvertFuncs[354] = function(event) {
	return { name: 'title_return' };
};

/*******************************************************************************
 * システム設定
 ******************************************************************************/
// -----------------------------------------------------------------------------
// @battle_bgm/132:戦闘BGMの変更
// -----------------------------------------------------------------------------
module.validates['battle_bgm'] = {
	volume: {isNumeric: [0, 100]},
	pitch: {isNumeric: [50, 150]},
	pan: {isNumeric: [-100, 100]}
};
module.convertFuncs['battle_bgm'] = function(args) {
	return {
		code: 132,
		indent: this._indent,
		parameters: [
			{
				name: args['name'] || '',
				volume: parseInt(args['volume'] || 90, 10),
				pitch: parseInt(args['pitch'] || 100, 10),
				pan: parseInt(args['pan'] || 0, 10)
			}
		]
	}
};
module.deconvertFuncs[132] = function(event) {
	var args = {};

	this.deconvertAudio('name', 'volume', 'pitch', 'pan', args, event.parameters[0], '');

	return { name: 'battle_bgm', args: args };
};

// -----------------------------------------------------------------------------
// @battle_win_me/133:勝利MEの変更
// -----------------------------------------------------------------------------
module.validates['battle_win_me'] = {
	volume: {isNumeric: [0, 100]},
	pitch: {isNumeric: [50, 150]},
	pan: {isNumeric: [-100, 100]}
}
module.convertFuncs['battle_win_me'] = function(args) {
	return {
		code: 133,
		indent: this._indent,
		parameters: [
			{
				name: args['name'] || '',
				volume: parseInt(args['volume'] || 90, 10),
				pitch: parseInt(args['pitch'] || 100, 10),
				pan: parseInt(args['pan'] || 0, 10)
			}
		]
	}
};
module.deconvertFuncs[133] = function(event) {
	var args = {};

	this.deconvertAudio('name', 'volume', 'pitch', 'pan', args, event.parameters[0], '');

	return { name: 'battle_win_me', args: args };
};

// -----------------------------------------------------------------------------
// @battle_loss_me/139:敗北MEの変更
// -----------------------------------------------------------------------------
module.validates['battle_loss_me'] = {
	volume: {isNumeric: [0, 100]},
	pitch: {isNumeric: [50, 150]},
	pan: {isNumeric: [-100, 100]}
}
module.convertFuncs['battle_loss_me'] = function(args) {
	return {
		code: 139,
		indent: this._indent,
		parameters: [
			{
				name: args['name'] || '',
				volume: parseInt(args['volume'] || 90, 10),
				pitch: parseInt(args['pitch'] || 100, 10),
				pan: parseInt(args['pan'] || 0, 10)
			}
		]
	}
};
module.deconvertFuncs[139] = function(event) {
	var args = {};

	this.deconvertAudio('name', 'volume', 'pitch', 'pan', args, event.parameters[0], '');

	return { name: 'battle_loss_me', args: args };
};

// -----------------------------------------------------------------------------
// @vehicle_bgm/140:乗り物BGMの変更
// -----------------------------------------------------------------------------
module.validates['vehicle_bgm'] = {
	vehicle: {
		notEmpty: [],
		list: [
			'0', '1', '2',
			'boat', 'ship', 'airship'
		]
	},
	volume: {isNumeric: [0, 100]},
	pitch: {isNumeric: [50, 150]},
	pan: {isNumeric: [-100, 100]}
};
module.convertFuncs['vehicle_bgm'] = function(args) {
	var vehicle = this.convertList(args['vehicle'], {
		'0': 0,
		'1': 1,
		'2': 2,
		'boat': 0,
		'ship': 1,
		'airship': 2
	});

	return {
		code: 140,
		indent: this._indent,
		parameters: [
			vehicle,
			{
				name: args['name'] || '',
				volume: parseInt(args['volume'] || 90, 10),
				pitch: parseInt(args['pitch'] || 100, 10),
				pan: parseInt(args['pan'] || 0, 10)
			}
		]
	}
};
module.deconvertFuncs[140] = function(event) {
	var args = {};

	this.deconvertList('vehicle', args, event.parameters[0], [
		0, 'boat',
		1, 'ship',
		2, 'airship'
	]);
	this.deconvertAudio('name', 'volume', 'pitch', 'pan', args, event.parameters[1]);

	return { name: 'vehicle_bgm', args: args };
};

// -----------------------------------------------------------------------------
// @save_disable/134:セーブ禁止の変更
// -----------------------------------------------------------------------------
module.validates['save_disable'] = {
	flag: {list: flagList}
};
module.convertFuncs['save_disable'] = function(args) {
	return {
		code: 134,
		indent: this._indent,
		parameters: [
			this.convertFlag(args['flag'], 0)
		]
	}
};
module.deconvertFuncs[134] = function(event) {
	var args = {};

	this.deconvertFlag('flag', args, event.parameters[0]);

	return { name: 'save_disable', args: args };
};

// -----------------------------------------------------------------------------
// @menu_disable/135:メニュー禁止の変更
// -----------------------------------------------------------------------------
module.validates['menu_disable'] = {
	flag: {list: flagList}
};
module.convertFuncs['menu_disable'] = function(args) {
	return {
		code: 135,
		indent: this._indent,
		parameters: [
			this.convertFlag(args['flag'], 0)
		]
	}
};
module.deconvertFuncs[135] = function(event) {
	var args = {};

	this.deconvertFlag('flag', args, event.parameters[0]);

	return { name: 'menu_disable', args: args };
};

// -----------------------------------------------------------------------------
// @encount_disable/136:エンカウント禁止の変更
// -----------------------------------------------------------------------------
module.validates['encount_disable'] = {
	flag: {list: flagList}
};
module.convertFuncs['encount_disable'] = function(args) {
	return {
		code: 136,
		indent: this._indent,
		parameters: [
			this.convertFlag(args['flag'], 0)
		]
	}
};
module.deconvertFuncs[136] = function(event) {
	var args = {};

	this.deconvertFlag('flag', args, event.parameters[0]);

	return { name: 'encount_disable', args: args };
};

// -----------------------------------------------------------------------------
// @formation_disable/137:並び替え禁止の変更
// -----------------------------------------------------------------------------
module.validates['formation_disable'] = {
	flag: {list: flagList}
};
module.convertFuncs['formation_disable'] = function(args) {
	return {
		code: 137,
		indent: this._indent,
		parameters: [
			this.convertFlag(args['flag'], 0)
		]
	}
};
module.deconvertFuncs[137] = function(event) {
	var args = {};

	this.deconvertFlag('flag', args, event.parameters[0]);

	return { name: 'formation_disable', args: args };
};

// -----------------------------------------------------------------------------
// @window_color/138:ウィンドウカラーの変更
// -----------------------------------------------------------------------------
module.validates['window_color'] = {
	red: {isNumeric: [-255, 255]},
	blue: {isNumeric: [-255, 255]},
	green: {isNumeric: [-255, 255]}
};
module.convertFuncs['window_color'] = function(args) {
	return {
		code: 138,
		indent: this._indent,
		parameters: [
			[
				parseFloat(args['red'] || 0.0),
				parseFloat(args['green'] || 0.0),
				parseFloat(args['blue'] || 0.0),
				0
			]
		]
	}
};
module.deconvertFuncs[138] = function(event) {
	var args = {};

	this.deconvertColor('red', 'green', 'blue', null, args, event.parameters[0]);

	return { name: 'window_color', args: args };
};

// -----------------------------------------------------------------------------
// @actor_graphic/322:アクターの画像変更
// -----------------------------------------------------------------------------
module.validates['actor_graphic'] = {
	actor: {
		notEmpty: [],
		isNumeric: [1, null]
	},
	walk_index: {isNumeric: [0, 7]},
	face_index: {isNumeric: [0, 7]}
};
module.convertFuncs['actor_graphic'] = function(args) {
	return {
		code: 322,
		indent: this._indent,
		parameters: [
			parseInt(args['actor'], 10),
			args['walk_name'] || '',
			parseInt(args['walk_index'] || 0, 10),
			args['face_name'] || '',
			parseInt(args['face_index'] || 0, 10),
			args['battler_name'] || ''
		]
	}
};
module.deconvertFuncs[322] = function(event) {
	var args = {};

	this.deconvertNumeric('actor', args, event.parameters[0]);
	if (event.parameters[1]) {
		this.deconvertString('walk_name', args, event.parameters[1]);
		this.deconvertNumeric('walk_index', args, event.parameters[2]);
	}
	if (event.parameters[3]) {
		this.deconvertString('face_name', args, event.parameters[3]);
		this.deconvertNumeric('face_index', args, event.parameters[4]);
	}
	if (event.parameters[5]) {
		this.deconvertString('battler_name', args, event.parameters[5]);
	}

	return { name: 'actor_graphic', args: args };
};

// -----------------------------------------------------------------------------
// @vehicle_graphic/323:乗り物の画像変更
// -----------------------------------------------------------------------------
module.validates['vehicle_graphic'] = {
	vehicle: {
		notEmpty: [],
		list: [
			'0', '1', '2',
			'boat', 'ship', 'airship'
		]
	},
	index: {isNumeric: [0, 7]}
};
module.convertFuncs['vehicle_graphic'] = function(args) {
	var vehicle = this.convertList(args['vehicle'], {
		'0': 0,
		'1': 1,
		'2': 2,
		'boat': 0,
		'ship': 1,
		'airship': 2
	});

	return {
		code: 323,
		indent: this._indent,
		parameters: [
			vehicle,
			args['name'] || '',
			parseInt(args['index'] || 0, 10)
		]
	}
};
module.deconvertFuncs[323] = function(event) {
	var args = {};
	this.deconvertList('vehicle', args, event.parameters[0], [
		0, 'boat',
		1, 'ship',
		2, 'airship'
	]);

	if (event.parameters[1]) {
		this.deconvertString('name', args, event.parameters[1]);
		this.deconvertNumeric('index', args, event.parameters[2]);
	}

	return { name: 'vehicle_graphic', args: args };
};

/*******************************************************************************
 * マップ
 ******************************************************************************/
// -----------------------------------------------------------------------------
// @map_name_disable/281:マップ名表示の変更
// -----------------------------------------------------------------------------
module.validates['map_name_disable'] = {
	flag: {list: flagList}
};
module.convertFuncs['map_name_disable'] = function(args) {
	return {
		code: 281,
		indent: this._indent,
		parameters: [
			this.convertFlag(args['flag'], 1) == 0 ? 1 : 0
		]
	}
};
module.deconvertFuncs[281] = function(event) {
	var args = {};

	this.deconvertFlag('flag', args, event.parameters[0]);

	return { name: 'map_name_disable', args: args };
};

// -----------------------------------------------------------------------------
// @tileset/282:タイルセットの変更
// -----------------------------------------------------------------------------
module.validates['tileset'] = {
	id: {
		notEmpty: [],
		isNumeric: [1, null]
	}
};
module.convertFuncs['tileset'] = function(args) {
	return {
		code: 282,
		indent: this._indent,
		parameters: [
			parseInt(args['id'] || 0, 10)
		]
	}
};
module.deconvertFuncs[282] = function(event) {
	var args = {};

	this.deconvertNumeric('id', args, event.parameters[0]);

	return { name: 'tileset', args: args };
};

// -----------------------------------------------------------------------------
// @battle_background/283:戦闘背景の変更
// -----------------------------------------------------------------------------
module.convertFuncs['battle_background'] = function(args) {
	return {
		code: 283,
		indent: this._indent,
		parameters: [
			args['floor'] || '',
			args['wall'] || ''
		]
	}
};
module.deconvertFuncs[283] = function(event) {
	var args = {};

	this.deconvertString('floor', args, event.parameters[0], '');
	this.deconvertString('wall', args, event.parameters[1], '');

	return { name: 'battle_background', args: args };
};

// -----------------------------------------------------------------------------
// @parallax/284:遠景の変更
// -----------------------------------------------------------------------------
module.validates['parallax'] = {
	loop_x: {isBool: []},
	scroll_x: {isNumeric: [-32, 32]},
	loop_y: {isBool: []},
	scroll_y: {isNumeric: [-32, 32]}
};
module.convertFuncs['parallax'] = function(args) {
	var loopX = (args['loop_x'] || 'false') == 'true' ? true : false;
	var loopY = (args['loop_y'] || 'false') == 'true' ? true : false;

	return {
		code: 284,
		indent: this._indent,
		parameters: [
			args['name'] || '',
			loopX,
			loopY,
			loopX ? parseInt(args['scroll_x'] || 0, 10) : 0,
			loopY ? parseInt(args['scroll_y'] || 0, 10) : 0
		]
	}
};
module.deconvertFuncs[284] = function(event) {
	var args = {};

	this.deconvertString('name', args, event.parameters[0], '');
	this.deconvertBoolean('loop_x', args, event.parameters[1]);
	this.deconvertBoolean('loop_y', args, event.parameters[2]);
	this.deconvertNumeric('scroll_x', args, event.parameters[3]);
	this.deconvertNumeric('scroll_y', args, event.parameters[4]);

	return { name: 'parallax', args: args };
};

// -----------------------------------------------------------------------------
// @pos_info/285:指定位置の情報取得
// -----------------------------------------------------------------------------
module.validates['pos_info'] = {
	var: {
		notEmpty: [],
		isNumeric: [1, null]
	},
	type: {
		list: [
			'0', '1', '2', '3', '4', '5', '6',
			'terrain', 'event', 'tile1', 'tile2', 'tile3', 'tile4', 'region'
		]
	},
	pos: {
		list: [
			'0', '1',
			'const', 'var'
		]
	},
	x: {
		notEmpty: [],
		isNumeric: [0, null],
		varCheck2: ['post']
	},
	y: {
		notEmpty: [],
		isNumeric: [0, null],
		varCheck2: ['post']
	}
};
module.convertFuncs['pos_info'] = function(args) {
	var loopX = (args['loop_x'] || 'false') == 'true' ? true : false;
	var loopY = (args['loop_y'] || 'false') == 'true' ? true : false;
	var type = this.convertList(args['type'], {
		'0': 0,
		'1': 1,
		'2': 2,
		'3': 3,
		'4': 4,
		'5': 5,
		'6': 6,
		'terrain': 0,
		'event': 1,
		'tile1': 2,
		'tile2': 3,
		'tile3': 4,
		'tile4': 5,
		'region': 6
	}, 0);
	var pos = this.convertList(args['pos'], {
		'0': 0,
		'1': 1,
		'const': 0,
		'var': 1
	}, 0);

	return {
		code: 285,
		indent: this._indent,
		parameters: [
			parseInt(args['var'], 10),
			type,
			pos,
			parseInt(args['x'], 10),
			parseInt(args['y'], 10)
		]
	}
};
module.deconvertFuncs[285] = function(event) {
	var args = {};

	this.deconvertList('type', args, event.parameters[1], [
		0, 'terrain',
		1, 'event',
		2, 'tile1',
		3, 'tile2',
		4, 'tile3',
		5, 'tile4',
		6, 'region'
	]);
	this.deconvertList('pos', args, event.parameters[2], [
		0, 'const',
		1, 'var'
	]);
	this.deconvertNumeric('var', args, event.parameters[0]);
	this.deconvertNumeric('x', args, event.parameters[2]);
	this.deconvertNumeric('y', args, event.parameters[3]);

	return { name: 'pos_info', args: args };
};

/*******************************************************************************
 * バトル
 ******************************************************************************/
// -----------------------------------------------------------------------------
// @enemy_hp/331:敵キャラのHP増減
// -----------------------------------------------------------------------------
module.validates['enemy_hp'] = {
	enemy: {
		notEmpty: [],
		isNumeric: [-1, null]
	},
	value: {
		notEmpty: [],
		regCheck: varRegCheck,
		varCheck: varVarCheck
	},
	death: {isBool: []}
};
module.convertFuncs['enemy_hp'] = function(args) {
	var params = {value: args['value']};

	convertVarType(params);
	convertVerOperate(params);

	return {
		code: 331,
		indent: this._indent,
		parameters: [
			parseInt(args['enemy'], 10),
			params.operate,
			params.type,
			parseInt(params.value, 10),
			(args['death'] || 'false') == 'true' ? true : false
		]
	}
};
module.deconvertFuncs[331] = function(event) {
	var args = {};

	this.deconvertNumeric('enemy', args, event.parameters[0]);
	this.deconvertNumeric('value', args, event.parameters[3]);
	deconvertVarOperate('value', args, event.parameters[1]);
	deconvertVarType('value', args, event.parameters[2]);
	this.deconvertBoolean('death', args, event.parameters[4]);

	return { name: 'enemy_hp', args: args };
};

// -----------------------------------------------------------------------------
// @enemy_mp/332:敵キャラのMP増減
// -----------------------------------------------------------------------------
module.validates['enemy_mp'] = {
	enemy: {
		notEmpty: [],
		isNumeric: [-1, null]
	},
	value: {
		notEmpty: [],
		regCheck: varRegCheck,
		varCheck: varVarCheck
	}
};
module.convertFuncs['enemy_mp'] = function(args) {
	var params = {value: args['value']};

	convertVarType(params);
	convertVerOperate(params);

	return {
		code: 332,
		indent: this._indent,
		parameters: [
			parseInt(args['enemy'], 10),
			params.operate,
			params.type,
			parseInt(params.value, 10)
		]
	}
};
module.deconvertFuncs[332] = function(event) {
	var args = {};

	this.deconvertNumeric('enemy', args, event.parameters[0]);
	this.deconvertNumeric('value', args, event.parameters[3]);
	deconvertVarOperate('value', args, event.parameters[1]);
	deconvertVarType('value', args, event.parameters[2]);

	return { name: 'enemy_mp', args: args };
};

// -----------------------------------------------------------------------------
// @enemy_tp/342:敵キャラのTP増減
// -----------------------------------------------------------------------------
module.validates['enemy_tp'] = {
	enemy: {
		notEmpty: [],
		isNumeric: [-1, null]
	},
	value: {
		notEmpty: [],
		regCheck: varRegCheck,
		varCheck: varVarCheck
	}
};
module.convertFuncs['enemy_tp'] = function(args) {
	var params = {value: args['value']};

	convertVarType(params);
	convertVerOperate(params);

	return {
		code: 342,
		indent: this._indent,
		parameters: [
			parseInt(args['enemy'], 10),
			params.operate,
			params.type,
			parseInt(params.value, 10)
		]
	}
};
module.deconvertFuncs[342] = function(event) {
	var args = {};

	this.deconvertNumeric('enemy', args, event.parameters[0]);
	this.deconvertNumeric('value', args, event.parameters[3]);
	deconvertVarOperate('value', args, event.parameters[1]);
	deconvertVarType('value', args, event.parameters[2]);

	return { name: 'enemy_tp', args: args };
};

// -----------------------------------------------------------------------------
// @enemy_state/333:敵キャラのステート変更
// -----------------------------------------------------------------------------
module.validates['enemy_state'] = {
	enemy: {
		notEmpty: [],
		isNumeric: [-1, null]
	},
	value: {
		notEmpty: [],
		isNumeric: [null, null],
		zeroCheck: []
	}
};
module.convertFuncs['enemy_state'] = function(args) {
	var params = {value: args['value']};

	convertVerOperate(params);

	return {
		code: 333,
		indent: this._indent,
		parameters: [
			parseInt(args['enemy'], 10),
			params.operate,
			parseInt(params.value, 10)
		]
	}
};
module.deconvertFuncs[333] = function(event) {
	var args = {};

	this.deconvertNumeric('enemy', args, event.parameters[0]);
	this.deconvertNumeric('value', args, event.parameters[2]);
	deconvertVarOperate('value', args, event.parameters[1]);

	return { name: 'enemy_state', args: args };
};

// -----------------------------------------------------------------------------
// @enemy_all_recovery/334:敵キャラの全回復
// -----------------------------------------------------------------------------
module.validates['enemy_all_recovery'] = {
	enemy: {
		notEmpty: [],
		isNumeric: [-1, null]
	}
};
module.convertFuncs['enemy_all_recovery'] = function(args) {
	return {
		code: 334,
		indent: this._indent,
		parameters: [
			parseInt(args['enemy'], 10)
		]
	}
};
module.deconvertFuncs[334] = function(event) {
	var args = {};

	this.deconvertNumeric('enemy', args, event.parameters[0]);

	return { name: 'enemy_all_recovery', args: args };
};

// -----------------------------------------------------------------------------
// @enemy_appear/335:敵キャラの出現
// -----------------------------------------------------------------------------
module.validates['enemy_appear'] = {
	enemy: {
		notEmpty: [],
		isNumeric: [0, null]
	}
};
module.convertFuncs['enemy_appear'] = function(args) {
	return {
		code: 335,
		indent: this._indent,
		parameters: [
			parseInt(args['enemy'], 10)
		]
	}
};
module.deconvertFuncs[335] = function(event) {
	var args = {};

	this.deconvertNumeric('enemy', args, event.parameters[0]);

	return { name: 'enemy_appear', args: args };
};

// -----------------------------------------------------------------------------
// @enemy_trans/336:敵キャラの変身
// -----------------------------------------------------------------------------
module.validates['enemy_trans'] = {
	enemy: {
		notEmpty: [],
		isNumeric: [0, null]
	},
	trans: {
		notEmpty: [],
		isNumeric: [1, null]
	}
};
module.convertFuncs['enemy_trans'] = function(args) {
	return {
		code: 336,
		indent: this._indent,
		parameters: [
			parseInt(args['enemy'], 10),
			parseInt(args['trans'], 10)
		]
	}
};
module.deconvertFuncs[336] = function(event) {
	var args = {};

	this.deconvertNumeric('enemy', args, event.parameters[0]);
	this.deconvertNumeric('trans', args, event.parameters[1]);

	return { name: 'enemy_trans', args: args };
};

// -----------------------------------------------------------------------------
// @battle_anime/337:戦闘アニメーションの表示
// -----------------------------------------------------------------------------
module.validates['battle_anime'] = {
	enemy: {
		notEmpty: [],
		isNumeric: [0, null]
	},
	anime: {
		notEmpty: [],
		isNumeric: [1, null]
	}
};
module.convertFuncs['battle_anime'] = function(args) {
	return {
		code: 337,
		indent: this._indent,
		parameters: [
			parseInt(args['enemy'], 10),
			parseInt(args['anime'], 10)
		]
	}
};
module.deconvertFuncs[337] = function(event) {
	var args = {};

	this.deconvertNumeric('enemy', args, event.parameters[0]);
	this.deconvertNumeric('anime', args, event.parameters[1]);

	return { name: 'battle_anime', args: args };
};

// -----------------------------------------------------------------------------
// @force/339:戦闘行動の強制
// -----------------------------------------------------------------------------
module.validates['force'] = {
	battler: {
		notEmpty: [],
		list: [
			'0', '1',
			'enemy', 'actor'
		]
	},
	id: {
		notEmpty: [],
		isNumeric: [0, null]
	},
	skill: {
		notEmpty: [],
		isNumeric: [1, null]
	},
	target: {
		notEmpty: [],
		isNumeric: [-2, null]
	}
};
module.convertFuncs['force'] = function(args) {
	var battler = this.convertList(args['battler'], {
		'0': 0,
		'1': 1,
		'enemy': 0,
		'actor': 1
	});

	return {
		code: 339,
		indent: this._indent,
		parameters: [
			battler,
			parseInt(args['id'], 10),
			parseInt(args['skill'], 10),
			parseInt(args['target'], 10)
		]
	}
};
module.deconvertFuncs[336] = function(event) {
	var args = {};

	this.deconvertList('battler', args, event.parameters[0], [
		0, 'enemy',
		1, 'actor'
	]);
	this.deconvertNumeric('id', args, event.parameters[1]);
	this.deconvertNumeric('skill', args, event.parameters[2]);
	this.deconvertNumeric('target', args, event.parameters[3]);

	return { name: 'force', args: args };
};

// -----------------------------------------------------------------------------
// @battle_abort/340:バトルの中断
// -----------------------------------------------------------------------------
module.convertFuncs['battle_abort'] = function(args) {
	return {
		code: 340,
		indent: this._indent,
		parameters: []
	}
};
module.deconvertFuncs[340] = function(event) {
	return { name: 'battle_abort' };
};

/*******************************************************************************
 * 上級
 ******************************************************************************/
// -----------------------------------------------------------------------------
// @script/355:スクリプト(ヘッダ)＆655:スクリプト
// -----------------------------------------------------------------------------
module.validates['script'] = { };
module.validates['script2'] = { };
module.splitFuncs['script'] = function(text, tag) {
	var scripts = [];
	this._index++;

	while(!this.eoc()) {
		text = this.getLine();
		if (this.getCommandName(text) == 'script_end') break;

		scripts.push(
			this.createData.call(
				this, 'script2', {
					value: text
				}
			)
		);

		this._index++;
	}

	if (this.eoc()) {
		throw new Tes.SyntaxError(
			'スクリプトタグが閉じられていません。',
			'syntax.script.footer.notFound.error'
		);
	}

	if (scripts.length > 0) {
		// 先頭のタグ名修正
		scripts[0].tag = 'script';

		for (var i = 0; i < scripts.length; i++) {
			this._data.push(scripts[i]);
		}
	}
};
module.convertFuncs['script'] = function(args) {
	return {
		code: 355,
		indent: this._indent,
		parameters: [args['value']]
	}
};
module.convertFuncs['script2'] = function(args) {
	return {
		code: 655,
		indent: this._indent,
		parameters: [args['value']]
	}
};
module.deconvertFuncs[355] = function(event) {
	var args = {};

	this.deconvertString('value', args, event.parameters[0]);

	return { name: 'script', args: args };
};
module.deconvertFuncs[655] = module.deconvertFuncs[355];

// -----------------------------------------------------------------------------
// @script_end/---:スクリプトの終了位置
// -----------------------------------------------------------------------------
module.splitFuncs['script_end'] = function(text, tag) {
	throw new Tes.SyntaxError(
		'スクリプトタグが開始されていません。',
		'syntax.script.header.notFound.error'
	);
};

// -----------------------------------------------------------------------------
// @plugin/356:プラグインコマンド
// -----------------------------------------------------------------------------
module.validates['plugin'] = {
	value: {notEmpty: []}
};
module.convertFuncs['plugin'] = function(args) {
	return {
		code: 356,
		indent: this._indent,
		parameters: [args['value']]
	};
};
module.deconvertFuncs[356] = function(event) {
	var args = {};

	this.deconvertString('value', args, event.parameters[0]);

	return { name: 'plugin', args: args }
};

/*******************************************************************************
 * その他
 ******************************************************************************/
// -----------------------------------------------------------------------------
// @end/0:イベントの終了位置
// -----------------------------------------------------------------------------
module.convertFuncs['end'] = function(args) {
	this._indent--;

	return {
		code: 0,
		indent: this._indent + 1,
		parameters: []
	}
};
module.deconvertFuncs[0] = function(event) {
	var args = {};

	return { name: 'end', indent: event.indent - 1 };
};

/*******************************************************************************
 * リンク関数
 ******************************************************************************/
// -----------------------------------------------------------------------------
// マップイベント
// -----------------------------------------------------------------------------
module.linkFuncs['rpgmv-map'] = function(target, targetData, eventList) {
	// パラメータチェック
	var attributeError = function(paramName) {
		return new Tes.LinkError(
			'[' + paramName + '] : リンク対象パラメータが未指定です。',
			'linker.attribute.notFound.error', [
				paramName
			]
		);
	};
	if (target.attrs.id == null) {
		throw attributeError('id');
	}
	if (target.attrs.page == null) {
		throw attributeError('page');
	}

	// 更新対象チェック
	var targetError = function() {
		return new Tes.LinkError(
			'更新対象が見つかりません。',
			'linker.target.notFound.error'
		);
	};
	var events = targetData.events;
	if (events == null) {
		throw targetError();
	} else if (!isArray(events)) {
		throw targetError();
	}

	var event = null;
	for (var i = 1; i < events.length; i++) {
		var e = events[i];

		if (e.id == target.attrs.id) {
			event = e;
			break;
		}
	}

	if (event == null) {
		throw targetError();
	}

	if (event.pages == null) {
		throw targetError();
	} else if (!isArray(event.pages)) {
		throw targetError();
	}

	var page = event.pages[target.attrs.page];
	if (page == null) {
		throw targetError();
	}

	if (page.list == null) {
		throw targetError();
	}else if (!isArray(page.list)) {
		throw targetError();
	}

	// イベントリストをセット
	page.list = eventList;

	return targetData;
};

// -----------------------------------------------------------------------------
// 敵グループイベント
// -----------------------------------------------------------------------------
module.linkFuncs['rpgmv-troop'] = function(target, targetData, eventList) {
	// パラメータチェック
	var attributeError = function(paramName) {
		return new Tes.LinkError(
			'[' + paramName + '] : リンク対象パラメータが未指定です。',
			'linker.attribute.notFound.error', [
				paramName
			]
		);
	};
	if (target.attrs.id == null) {
		throw attributeError('id');
	}
	if (target.attrs.page == null) {
		throw attributeError('page');
	}

	// 更新対象チェック
	var targetError = function() {
		return new Tes.LinkError(
			'更新対象が見つかりません。',
			'linker.target.notFound.error'
		);
	};

	var troops = targetData;
	if (!isArray(troops)) {
		throw targetError();
	}

	var troop = null;
	for (var i = 1; i < troops.length; i++) {
		var t = troops[i];

		if (t.id == target.attrs.id) {
			troop = t;
			break;
		}
	}

	if (troop == null) {
		throw targetError();
	}

	if (troop.pages == null) {
		throw targetError();
	} else if (!isArray(troop.pages)) {
		throw targetError();
	}

	var page = troop.pages[target.attrs.page];
	if (page == null) {
		throw targetError();
	}

	if (page.list == null) {
		throw targetError();
	}else if (!isArray(page.list)) {
		throw targetError();
	}

	// イベントリストをセット
	page.list = eventList;

	return targetData;
};

// -----------------------------------------------------------------------------
// コモンイベント
// -----------------------------------------------------------------------------
module.linkFuncs['rpgmv-common'] = function(target, targetData, eventList) {
	// パラメータチェック
	var attributeError = function(paramName) {
		return new Tes.LinkError(
			'[' + paramName + '] : リンク対象パラメータが未指定です。',
			'linker.attribute.notFound.error', [
				paramName
			]
		);
	};
	if (target.attrs.id == null) {
		throw attributeError('id');
	}

	// 更新対象チェック
	var targetError = function() {
		return new Tes.LinkError(
			'更新対象が見つかりません。',
			'linker.target.notFound.error'
		);
	};

	var commonEvents = targetData;
	if (!isArray(commonEvents)) {
		throw targetError();
	}

	var commonEvent = null;
	for (var i = 1; i < commonEvents.length; i++) {
		var e = commonEvents[i];

		if (e.id == target.attrs.id) {
			commonEvent = e;
			break;
		}
	}

	if (commonEvent == null) {
		throw targetError();
	}

	if (commonEvent.list == null) {
		throw targetError();
	}else if (!isArray(commonEvent.list)) {
		throw targetError();
	}

	// イベントリストをセット
	commonEvent.list = eventList;

	return targetData;
};

/*******************************************************************************
 * コンバート後処理
 ******************************************************************************/
module.postProcessors['converter'] = function() {
	this._index = 0;

	while (this._data.length > this._index) {
		// ルートタグのダミーをヘッダに収納
		if (this._data[this._index].code == 205) {
			var hIndex = this._index;
			var route = [];

			while (this._data[this._index + 1] && this._data[this._index + 1].code == 505) {
				route.push(this._data[++this._index].parameters[0]);
			}

			route.push({code: 0, parameters: []});
			this._data[hIndex].parameters[1].list = route;
		}

		// ショップタグの先頭をヘッダコードに変換
		if (this._data[this._index].code == 605) {
			this._data[this._index].code = 302;
			while (this._data[this._index + 1] && this._data[this._index + 1].code == 605) {
				this._index++;
			}
		}

		this._index++;
	}
};
module.postProcessors['deconverter'] = function() {
	this._index = 0;
	while (this._data.length > this._index) {
		if (typeof(this._data[this.index]) != 'string') {
			this._index++;
			continue;
		}

		var name = this._data[this._index].name;

		switch (name) {
			case 'comment':
			case 'scroll':
				while (this._data[this._index + 1] && this._data[this._index + 1].name != name) {
					this._index++;
				}
				this._data.splice(this._index++, 0, { name: name + '_end' });

				break;
		}

		this._index++;
	}

	this._index = 0;
	while (this._data.length > this._index) {
		if (typeof(this._data) != 'string') {
			this._index++;
			continue;
		}
		var event = this._data[this._index];

		switch (event.name) {
			case 'script':
				this._data.splice(this._index++, 0, { name: name + 'script' });
				while (this._data[this._index].name != 'script') {
					this._data[this._index] = this._data[this._index].value;

					this.index++;
				}
				this._data.splice(this._index++, 0, { name: name + 'script_end' });

				break;
		}

		this._index++;
	}
};

/**
 * デコンパイル対象の判定を行う。
 */
module.decompileTargetIdentifier = function(basename, data) {
	var linkData = {
		type: undefined,
		targets: []
	};

	if (isArray(data)) {
		if (data[0] == null && data[1] != null) {
			var item = data[1];

			if (item.members != null && item.pages != null && isArray(item.pages)) {
				// Troop
				linkData.type = 'rpgmv-troop';

				for (var i = 1; i < data.length; i++) {
					item = data[i];
					if (item == null) continue;

					for (var j = 0; j < item.pages.length; j++) {
						var page = item.pages[j];

						if (page.list != null && isArray(page.list)) {
							linkData.targets.push({
								name: basename + '_' + item.id.toString() + '_' + j.toString(),
								list: page.list,
								attrs: {
									id: item.id,
									page: j 
								}
							});
						}
					}
				}
			} else if (item.list != null && item.switchId != null) {
				// CommonEvents
				linkData.type = 'rpgmv-common';

				for (var i = 1; i < data.length; i++) {
					item = data[i];
					if (item == null) continue;

					linkData.targets.push({
						name: basename + '_' + item.id.toString(),
						list: item.list,
						attrs: {
							id: item.id
						}
					});
				}
			} else {
				return null;
			}

			return linkData;
		}
	} else if (data.data != null && isArray(data.data)) {
		if (data.events != null && isArray(data.events)) {
			// Map Events
			linkData.type = 'rpgmv-map';

			var targets = [];

			for (var i = 1; i < data.events.length; i++) {
				var event = data.events[i];
				if (event == null) continue;

				if (event.pages != null && isArray(event.pages)) {
					for (var j = 0; j < event.pages.length; j++) {
						var page = event.pages[j];

						if (page.list != null && isArray(page.list)) {
							linkData.targets.push({
								name: basename + '_' + event.id.toString() + '_' + j.toString(),
								list: page.list,
								attrs: {
									id: event.id,
									page: j 
								}
							});
						}
					}
				}
			}

			return linkData;
		}
	}

	return null;
};

module.stringResourceData.default = {
	"converter": {
		"choice": {
			"label.empty.error": "選択肢のラベルが未設定です。"
		},
		"var": {
			"value.zeroCheck.error": "{0} : 0以下 を代入することはできません。"
		}
	},
	"syntax": {
		"choice": {
			"tag.notFound.error": "選択肢分岐前にヘッダまたは終端タグがありません。"
		},
		"scroll": {
			"content.empty.error": "文章スクロールの中身がありません。"
		},
		"comment": {
			"footer.notFound.error": "コメントタグが閉じられていません。",
			"header.notFound.error": "コメントタグが開始されていません。"
		},
		"profile": {
			"footer.notFound.error": "プロフィールタグが閉じられていません。",
			"header.notFound.error": "プロフィールタグが開始されていません。"
		},
		"route": {
			"empty.error": "移動ルートが設定されていません。",
			"header.notFound.error": "移動ルートのヘッダがありません。",
			"command.notFound.error": "存在しない移動コマンドです。"
		},
		"battle": {
			"tag.notFound.error": "戦闘分岐前にヘッダまたは終端タグがありません。"
		},
		"script": {
			"footer.notFound.error": "スクリプトタグが閉じられていません。",
			"header.notFound.error": "スクリプトタグが開始されていません。"
		}
	},
	"deconverter": {
		"route": {
			"undefined.error": "ルートオブジェクトが未設定です。",
		}
	},
	"linker": {
		"target.notFound.error": "更新対象が見つかりません。",
		"attribute.notFound.error": "{0} : リンク対象パラメータが未指定です。"
	}
};

return module;
});
